
#ifndef object_spectrum_h
#define object_spectrum_h

#include "scene/scene.h"
#include "common90/d3d-util.h"

//#include "common/dxerr9.h"
#include "common/dxutil.h"
#include "common/d3dfont.h"

#define NUM_SECTORS 16
#define NUM_TRACKS 7
#define INNER_RADIUS 0.10f
#define WHEEL_RADIUS 1.0f
#define MIN_LEVEL 50.0f
#define MAX_LEVEL 128.0f
#define VERTS_PER_CUBE 20
#define TRIS_PER_CUBE 10
#define RENDER_PASSES 2
#define RADIANS_PER_SECTOR ((2.0f * D3DX_PI) / NUM_SECTORS)
#define SPACING_PER_SECTOR (D3DX_PI / 100.0f);

#define DISTANCE_PER_TRACK ((WHEEL_RADIUS-INNER_RADIUS) / NUM_TRACKS)
#define SPACING_PER_TRACK (WHEEL_RADIUS / 50.0f)

// A structure for our custom vertex type
struct SPECTRUMVERTEX
{
    FLOAT x, y, z;      // The untransformed, 3D position for the vertex
    DWORD color;        // The vertex color
    FLOAT u, v;         // Texture coordinates
};

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_SPECTRUMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)

class object_spectrum : public scene  
{
public:

    object_spectrum(char* opts = 0);
	
    virtual ~object_spectrum();
	scene* create(char* opts = 0);
    virtual HRESULT init(IDirect3DDevice9* device);
	virtual HRESULT restore(IDirect3DDevice9* device);
	virtual HRESULT compose(IDirect3DDevice9* device, DWORD t);
    

    virtual HRESULT render(  IDirect3DDevice9* device,
                        IDirect3DTexture9* texture );

	
	int handle(char* type, char* ev);

private:
	 VOID    CalculateColors();
    VOID    ShiftColor( DWORD& dwColor, int& offset );
    HRESULT RotateColors();

	int onetime;
    
    D3DXVECTOR3 m_vCameraPos;

    SPECTRUMVERTEX m_vertices[RENDER_PASSES][NUM_TRACKS][NUM_SECTORS][VERTS_PER_CUBE];
    FLOAT m_velocities[RENDER_PASSES][NUM_TRACKS][NUM_SECTORS];
    
    LPDIRECT3DVERTEXBUFFER9 m_pVB;
    LPDIRECT3DINDEXBUFFER9 m_pIB;

    D3DXCOLOR m_colorInner;
    D3DXCOLOR m_colorOuter;
    D3DXCOLOR m_colorsOn[NUM_TRACKS];
    
    LPDIRECT3DTEXTURE9 m_pCubeTex;


	LPDIRECT3D9       m_pD3D;              // The main D3D object
    LPDIRECT3DDEVICE9 m_pd3dDevice;        // The D3D rendering device
    D3DCAPS9          m_d3dCaps;           // Caps for the device
    D3DSURFACE_DESC   m_d3dsdBackBuffer;   // Surface desc of the backbuffer
    DWORD             m_dwCreateFlags;     // Indicate sw or hw vertex processing
    DWORD             m_dwWindowStyle;     // Saved window style for mode switches
    RECT              m_rcWindowBounds;    // Saved window bounds for mode switches
    RECT              m_rcWindowClient;    // Saved client area size for mode switches
    
    // Variables for timing
    FLOAT             m_fTime;             // Current time in seconds
    FLOAT             m_fElapsedTime;      // Time elapsed since last frame
    FLOAT             m_fFPS;              // Instanteous frame rate
    TCHAR             m_strDeviceStats[90];// String to hold D3D device stats
    TCHAR             m_strFrameStats[90]; // String to hold frame stats

    // Overridable variables for the app
    TCHAR*            m_strWindowTitle;    // Title for the app's window
    DWORD             m_dwCreationWidth;   // Width used to create window
    DWORD             m_dwCreationHeight;  // Height used to create window
    bool              m_bShowCursorWhenFullscreen; // Whether to show cursor when fullscreen
    bool              m_bClipCursorWhenFullscreen; // Whether to limit cursor pos when fullscreen
    bool              m_bStartFullscreen;  // Whether to start up the app in fullscreen mode
    bool              m_bCreateMultithreadDevice; // Whether to create a multithreaded device
    bool              m_bAllowDialogBoxMode; // If enabled the framework will try to enable GDI dialogs while in fullscreen

};
#endif 
