//------------------------------------------------------------------------------
// File: PlaneScene.cpp
//
// Desc: DirectShow sample code - implementation of the background_scene class
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------

#include <hush/source.h>

#include "background-scene.h"
#include "scene/util.h"

#include <D3DX9.h>
#include "common/D3DUtil.h"
#include "common/DXUtil.h"
//#include "D3DEnumeration.h"
//#include "D3DSettings.h"
//#include "D3DApp.h"
//#include "D3DFile.h"
//#include "D3DFont.h"


extern HINSTANCE hInst;
extern IDirect3D9* g_d3d; 
extern IDirect3DDevice9* g_d3dd;
extern HWND  g_hWnd;


const DWORD FVF = (D3DFVF_XYZRHW | D3DFVF_TEX1);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

background_scene::background_scene(TCHAR* s)
{
	
	image = 0;

	if (s) { // image
	    
	    image = s;
	}
}

background_scene::~background_scene()
{

}

HRESULT 
background_scene::init(IDirect3DDevice9* device)
{
    HRESULT hr = S_OK;


    m_vertexBuffer = NULL;
    if (image) {
	if( FAILED( D3DUtil_CreateTexture( device, image,
		&g_pBackgroundTexture) ) ) {
		//ASSERT(10!=10);
        return -1; //D3DAPPERR_MEDIANOTFOUND;
	}
	}
	


    m_time = GetTickCount();

    return hr;
}

HRESULT 
background_scene::restore(IDirect3DDevice9* device)
{
	HRESULT hr = S_OK;

	// get the difference in time
    //DWORD dwCurrentTime;
   // dwCurrentTime = GetTickCount();
   // double difference = m_time - dwCurrentTime ;
    
  

	D3DSURFACE_DESC    m_d3dsdBackBuffer;
	LPDIRECT3DSURFACE9 pBackBuffer;

	device->GetBackBuffer( 0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer );
	pBackBuffer->GetDesc( &m_d3dsdBackBuffer );
	pBackBuffer->Release();
 


		float depth = 2.0f; //1.0f;
	float xmag = 1.0f; //depth*1.5;
	float ymag = 1.0f; //depth*1.5;
	float xoff = 0.0f; //-depth*170.0f;
	float yoff = 0.0f; //-depth*170.0f;

	float tu = image?1.0f:_tu;
	float tv = image?1.0f:_tv;
	//float rhw = 1.0f;
    float rhw = 0.2f;
	
	BACKGROUNDVERTEX Vertices[] =
    {
		{  xoff+0.0f,  yoff+0.0f, depth, rhw, 0.0f, 0.0f, }, // x, y, z, rhw, tu, tv
		{  xoff+xmag*(float)m_d3dsdBackBuffer.Width,  yoff+0.0f, depth, rhw, tu, 0.0f, },
		{  xoff+xmag*(float)m_d3dsdBackBuffer.Width, yoff+ymag*(float)m_d3dsdBackBuffer.Height,
		    //0.9f, rhw, tu, tv, },
		    depth-0.1, rhw, tu, tv, },
		{  xoff+0.0f, yoff+ymag*(float)m_d3dsdBackBuffer.Height, depth, rhw, 0.0f,tv, },
    };

	m_sizeofVertices = sizeof(Vertices);

	// sixth parameter is new in DX9
	// Create a square for rendering the background
    if( FAILED( device->CreateVertexBuffer( m_sizeofVertices,
                                                  D3DUSAGE_WRITEONLY, FVF,
                                                  D3DPOOL_MANAGED, &m_vertexBuffer, NULL ) ) )
        return E_FAIL;

    BACKGROUNDVERTEX* pVertices;
	// the third parameter changed from BYTE** to VOID** in DX9
    if( FAILED( m_vertexBuffer->Lock( 0, m_sizeofVertices, (VOID**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, Vertices, m_sizeofVertices);
    m_vertexBuffer->Unlock();

	
	return hr;
}


HRESULT 
background_scene::render( IDirect3DDevice9* device, IDirect3DTexture9* texture ) 
{
    HRESULT hr = S_OK;
 
   //translation(0,0,-10);
   // translation(0,0,10);
    render_begin(device);
	if (_texture) texture = _texture;


		// Set up texture stage states for the background
		if (image)
	       device->SetTexture( 0, g_pBackgroundTexture );
		else device->SetTexture( 0, texture );
		device->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
		device->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );

		// Passing an FVF to IDirect3DDevice9::SetFVF specifies a legacy FVF with stream 0.
	    device->SetFVF( FVF ); 
		device->SetStreamSource( 0, m_vertexBuffer, 0, sizeof(BACKGROUNDVERTEX) );
	    device->DrawPrimitive( D3DPT_TRIANGLEFAN, 0, 2 );

			if (1) { //XX
    device->SetRenderState( D3DRS_ZENABLE,  D3DZB_TRUE );
    device->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
  
    device->SetRenderState( D3DRS_ALPHATESTENABLE,  FALSE );
	}

	if (1) { //XX
			hr = device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
    hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
    hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TEXTURE);
    hr = device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
		}

		render_end(device);

    return hr;
}

