//------------------------------------------------------------------------------
// File: Planecylinder_scene.cpp
//
// Desc: DirectShow sample code - implementation of the cylinder_scene class
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------


#include "cylinder-scene.h"

#define SHOW_HOW_TO_USE_TCI


//CMyD3DApplication* g_myapp;
extern HINSTANCE hInst;
extern IDirect3D9* g_d3d; 
extern IDirect3DDevice9* g_d3dd;
extern HWND  g_hWnd;

LPDIRECT3D9             g_pD3D       = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice = NULL; // Our rendering device
LPDIRECT3DVERTEXBUFFER9 g_pVB        = NULL; // Buffer to hold vertices
LPDIRECT3DTEXTURE9      g_pTexture   = NULL; // Our texture

float g_fTU; float g_fTV;

// A structure for our custom vertex type. We added texture coordinates
struct CUSTOMVERTEX
{
    D3DXVECTOR3 position; // The position
    D3DCOLOR    color;    // The color
#ifndef SHOW_HOW_TO_USE_TCI
    FLOAT       tu, tv;   // The texture coordinates
#endif
};

// Our custom FVF, which describes our custom vertex structure
#ifdef SHOW_HOW_TO_USE_TCI
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)
#else
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

cylinder_scene::cylinder_scene(int opt)
{

option = opt;
}

cylinder_scene::~cylinder_scene()
{

}

HRESULT 
cylinder_scene::init(IDirect3DDevice9* d3ddev)
{
    HRESULT hr = S_OK;

    //return 0;

	g_pd3dDevice = d3ddev;

    //if( ! d3ddev )
      //  return E_POINTER;

	 // Turn off culling
    g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    // Turn off D3D lighting
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

    // Turn on the zbuffer
    g_pd3dDevice->SetRenderState( D3DRS_ZENABLE, TRUE );

	  // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 50*2*sizeof(CUSTOMVERTEX),
                                                  0, D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_DEFAULT, &g_pVB, NULL ) ) )
    {
	    //ASSERT(0);
        return E_FAIL;
    }
//ASSERT(0);
//return 0;
    // Fill the vertex buffer. We are setting the tu and tv texture
    // coordinates, which range from 0.0 to 1.0
    CUSTOMVERTEX* pVertices;
    if( FAILED( g_pVB->Lock( 0, 0, (void**)&pVertices, 0 ) ) ) {
        //ASSERT(0);
        return E_FAIL;
	}
	//ASSERT(0);

		for( DWORD i=0; i<50; i++ )
    {
        FLOAT theta = (2*D3DX_PI*i)/(50-1);
        float factor = 0.35;
		//float factor = 1;
		float trans = 0.1;
        pVertices[2*i+0].position = D3DXVECTOR3( sinf(theta),-1.0f, cosf(theta) );
        pVertices[2*i+0].color    = 0xffffffff;
#ifndef SHOW_HOW_TO_USE_TCI
        //pVertices[2*i+0].tu       = g_fTU; // ((FLOAT)i)/(50-1);
		//pVertices[2*i+0].tv       =   1.0f;
        pVertices[2*i+0].tv       = g_fTV; // 1.0f;
		pVertices[2*i+0].tu       = trans +  factor * ((FLOAT)i)/(50-1);
        //pVertices[2*i+0].tv       =   1.0f;
#endif

        pVertices[2*i+1].position = D3DXVECTOR3( sinf(theta), 1.0f, cosf(theta) );
        pVertices[2*i+1].color    = 0xff808080;
#ifndef SHOW_HOW_TO_USE_TCI
        //pVertices[2*i+1].tu       = g_fTU; // ((FLOAT)i)/(50-1);
        //pVertices[2*i+1].tv       = g_fTV; //0.0f;
		pVertices[2*i+1].tu       =  trans + factor * ((FLOAT)i)/(50-1);
        pVertices[2*i+1].tv       = 0.0f;
#endif
    }
    g_pVB->Unlock();

return 0;
//ASSERT(0);



    return hr;
}

HRESULT 
cylinder_scene::restore(IDirect3DDevice9* d3ddev)
{
    HRESULT hr = S_OK;
	return hr;
}

HRESULT 
cylinder_scene::render( IDirect3DDevice9* device, IDirect3DTexture9* texture ) 
{
    HRESULT hr = S_OK;
g_pd3dDevice = device;
g_pTexture = texture;

static count = 0;
  
    render_begin(device);

    D3DXMATRIXA16 matWorld;
    
	D3DXMatrixIdentity( &matWorld );
    D3DXMatrixRotationX( &matWorld, timeGetTime()/1000.0f );
	if (option > 0) D3DXMatrixRotationZ( &matWorld, timeGetTime()/1000.0f );
	if (option > 1) D3DXMatrixRotationZ( &matWorld, -timeGetTime()/1000.0f );
    matWorld._43 = -1.50f;
	matWorld._42 = 1.00f;
    device->MultiplyTransform( D3DTS_WORLD, &matWorld );  // relative
	//device->SetTransform( D3DTS_WORLD, &matWorld );
    //device->SetTransform( D3DTS_PROJECTION, &matProj );

        // Setup our texture. Using textures introduces the texture stage states,
        // which govern how textures get blended together (in the case of multiple
        // textures) and lighting information. In this case, we are modulating
        // (blending) our texture with the diffuse color of the vertices.
        g_pd3dDevice->SetTexture( 0, g_pTexture );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );

    #ifdef SHOW_HOW_TO_USE_TCI
        // Note: to use D3D texture coordinate generation, use the stage state
        // D3DTSS_TEXCOORDINDEX, as shown below. In this example, we are using
        // the position of the vertex in camera space to generate texture
        // coordinates. The tex coord index (TCI) parameters are passed into a
        // texture transform, which is a 4x4 matrix which transforms the x,y,z
        // TCI coordinates into tu, tv texture coordinates.

        // In this example, the texture matrix is setup to 
        // transform the texture from (-1,+1) position coordinates to (0,1) 
        // texture coordinate space:
        //    tu =  0.5*x + 0.5
        //    tv = -0.5*y + 0.5
        D3DXMATRIXA16 mat;
		if (_embedded) {
        mat._11 = 0.25f; mat._12 = 0.00f; mat._13 = 0.00f; mat._14 = 0.00f;
        mat._21 = 0.00f; mat._22 =-0.25f; mat._23 = 0.00f; mat._24 = 0.00f;
        mat._31 = 0.00f; mat._32 = 0.00f; mat._33 = 1.00f; mat._34 = 0.00f;
        mat._41 = 0.50f; mat._42 = 0.50f; mat._43 = 0.00f; mat._44 = 1.00f;
		} else {
		/*
		mat._11 = 0.25; mat._12 = 0.00f; mat._13 = 0.00f; mat._14 = 0.00f;
        mat._21 = 0.00f; mat._22 = -0.25f; mat._23 = 0.00f; mat._24 = 0.00f;
        mat._31 = 0.00f; mat._32 = 0.00f; mat._33 = g_fTV;  mat._34 = 0.00f;
        mat._41 = 0.50 * g_fTU; mat._42 = 0.5 * g_fTV; mat._43 = 0.00f; mat._44 = 1;
*/
		mat._11 = 0.25*g_fTU; mat._12 = 0.00f; mat._13 = 0.00f; mat._14 = 0.00f;
        mat._21 = 0.00f; mat._22 = -0.25f*g_fTV; mat._23 = 0.00f; mat._24 = 0.00f;
        mat._31 = 0.00f; mat._32 = 0.00f; mat._33 = g_fTV;  mat._34 = 0.00f;
        mat._41 = 0.50 * g_fTU; mat._42 = 0.5 * g_fTV; mat._43 = 0.00f; mat._44 = g_fTU;
		}
        g_pd3dDevice->SetTransform( D3DTS_TEXTURE0, &mat );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, D3DTSS_TCI_CAMERASPACEPOSITION );
    #endif

        // Render the vertex buffer contents
        g_pd3dDevice->SetStreamSource( 0, g_pVB, 0, sizeof(CUSTOMVERTEX) );
        g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
        g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2*50-2 );


        // undo texture transforms
		//D3DXIdentityMatrix(&mat);
		//g_pd3dDevice->SetTransform( D3DTS_TEXTURE0, &mat); // 0
		//g_pd3dDevice->SetTransform( D3DTS_TEXTURE0, &mat );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, 0 );
        g_pd3dDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0);
 
		render_end(device);

    return hr;
}

void
cylinder_scene::rectangle( float fTU, float fTV )
{
	g_fTU = fTU;
	g_fTV = fTV;
 //ASSERT(0); 
}
