//------------------------------------------------------------------------------
// File: PlaneScene.cpp
//
// Desc: DirectShow sample code - implementation of the image_scene class
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------


#include "image-scene.h"
#include "scene/util.h"

#include <atlbase.h>

#include <D3DX9.h>
#include "common/D3DUtil.h"
#include "common/DXUtil.h"
//#include "D3DEnumeration.h"
//#include "D3DSettings.h"
//#include "D3DApp.h"
//#include "D3DFile.h"
//#include "D3DFont.h"


extern HINSTANCE hInst;
extern IDirect3D9* g_d3d; 
extern IDirect3DDevice9* g_d3dd;
extern HWND  g_hWnd;


//const DWORD FVF = (D3DFVF_XYZRHW | D3DFVF_TEX1);
const DWORD FVF = ( D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1 );

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

image_scene::image_scene(TCHAR* s)
{
	//state = 0;
	image = 0;

	if (s) { // image
	    //state = 1;
	    image = s;
	}

	m_vertices[0].position = CUSTOMVERTEX::Position(-1.0f,  1.0f, 0.0f); // top left
    m_vertices[1].position = CUSTOMVERTEX::Position(-1.0f, -1.0f, 0.0f); // bottom left
    m_vertices[2].position = CUSTOMVERTEX::Position( 1.0f,  1.0f, 0.0f); // top right
    m_vertices[3].position = CUSTOMVERTEX::Position( 1.0f, -1.0f, 0.0f); // bottom right

    // set up diffusion:
	/*
    m_vertices[0].color = 0xfffffff; // 0xffffffff;
    m_vertices[1].color = 0xfffffff; //0xff0000ff;
    m_vertices[2].color = 0xfffffff; //0xffffffff;
    m_vertices[3].color = 0xfffffff; // 0xff0000ff;
*/
	 m_vertices[0].color =  0xffffffff;
    m_vertices[1].color = 0xff0000ff;
    m_vertices[2].color = 0xffffffff;
    m_vertices[3].color = 0xff0000ff;


    // set up texture coordinates
    m_vertices[0].tu = 0.0f; m_vertices[0].tv = 0.0f; // low left
    m_vertices[1].tu = 0.0f; m_vertices[1].tv = 1.0f; // high left
    m_vertices[2].tu = 1.0f; m_vertices[2].tv = 0.0f; // low right
    m_vertices[3].tu = 1.0f; m_vertices[3].tv = 1.0f; // high right
}

image_scene::~image_scene()
{

}

HRESULT 
image_scene::init(IDirect3DDevice9* device)
{
    HRESULT hr = S_OK;


    m_vertexBuffer = NULL;


    hr = device->CreateVertexBuffer(sizeof(m_vertices),
		0,FVF,D3DPOOL_DEFAULT,& m_vertexBuffer.p, NULL );

	 
    if (image) {
	if( FAILED( D3DUtil_CreateTexture( device, image,
		&g_pBackgroundTexture) ) ) {
		//ASSERT(10!=10);
        return -1; // D3DAPPERR_MEDIANOTFOUND;
	}
	}
	
    m_time = GetTickCount();

    return hr;
}

HRESULT 
image_scene::restore(IDirect3DDevice9* device)
{
	HRESULT hr = S_OK;

	// get the difference in time
    DWORD dwCurrentTime;
    dwCurrentTime = GetTickCount();
    double difference = m_time - dwCurrentTime ;
	
    void* pData;
    FAIL_RET( m_vertexBuffer->Lock(0,sizeof(pData), &pData,0) );
    memcpy(pData,m_vertices,sizeof(m_vertices));  
	//ASSERT(0);
    FAIL_RET( m_vertexBuffer->Unlock() ); 

	return hr;
}


HRESULT 
image_scene::render( IDirect3DDevice9* device, IDirect3DTexture9* texture ) 
{
    HRESULT hr = S_OK;
 


	render_begin(device);



	// Set up texture stage states for the image
	if (image)
	       device->SetTexture( 0, g_pBackgroundTexture );
	else device->SetTexture( 0, texture );
	
		
	device->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	device->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );

		
    //FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE));
    //FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE));
    //FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_TEXTURE));
    //FAIL_RET(hr = device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE));


	FAIL_RET( device->SetFVF( FVF ) );
    FAIL_RET( device->SetStreamSource(0, m_vertexBuffer.p, 0,
		                              sizeof(image_scene::CUSTOMVERTEX)  ) ); 	
    FAIL_RET( device->DrawPrimitive(D3DPT_TRIANGLESTRIP,0,2) );  //draw quad 

	render_end(device);

    return hr;
}

void
image_scene::rectangle( float fTU, float fTV )
{
	if (!image) {
    m_vertices[0].tu = 0.0f; m_vertices[0].tv = 0.0f; // low left
    m_vertices[1].tu = 0.0f; m_vertices[1].tv = fTV;  // high left
    m_vertices[2].tu = fTU;  m_vertices[2].tv = 0.0f; // low right
    m_vertices[3].tu = fTU;  m_vertices[3].tv = fTV;  // high right
	}
}
