
#ifndef particle_skel_h
#define particle_skel_h

#include <hush/export.h>
#include "scene/scene.h"


#include <d3dx9.h>

#define u_int32 int
#define real32 float
/* Vertex definition*/
#define D3DFVF_ParticleVertex (D3DFVF_XYZ |  D3DFVF_DIFFUSE | D3DFVF_TEX1 )

/* Convert float value to DWORD */
inline unsigned long FtoDW(float val) { return *((unsigned long*)&val); }

/* Point Sprite structure*/
struct PointSprite
{
	FLOAT x, y, z;
	
	DWORD color;
	float u,v;
};

/* Particle structure */
struct Particle
{
	D3DXVECTOR3 m_vPosition;		/* Current position of particle */
	D3DXVECTOR3 m_vPrevPosition;	/* Last position of particle */
	D3DXVECTOR3 m_vVelocity;		/* Direction & speed */
	D3DXVECTOR3 m_vAcceleration;	/* Acceleration */

	FLOAT		m_fTotalEnergy;		/* Particle lifetime (in seconds) */
	FLOAT		m_fEnergy;			/* Energy left */
	FLOAT		m_fSize;			/* Size of the particle */
	FLOAT		m_fSizeDelta;		/* Change in size over time */
	FLOAT		m_fStartWeight;		/* Original weight */
	FLOAT		m_fWeight;			/* Determines effect of forces */
	FLOAT		m_fWeightDelta;		/* Change in weight */

	D3DXCOLOR	m_cStartColor;		/* Start color of the particle */
	D3DXCOLOR	m_cEndColor;		/* Final color of the particle */
};

struct ParticleInfo
{
	D3DXVECTOR3 m_vPositionVariation;		/* Variation in origin location */
	D3DXVECTOR3 m_vForceVariation;			/* Variation in force */
	D3DXVECTOR3 m_vStandardVelocity;		/* Standard velocity of emitted particles */
	D3DXVECTOR3 m_vVelocityVariation;		/* Variation in velocity */

	FLOAT		m_fStandardEnergy;			/* Standard lifetime of particle */
	FLOAT		m_fEnergyVariation;			/* Variation in lifetime */
	FLOAT		m_fSize;					/* Particle size */
	FLOAT		m_fStandardWeight;			/* Particle weight */
	FLOAT		m_fWeightVariation;			/* Variation in weights */
	FLOAT		m_fStandardWeightDelta;		/* Change in weight over time */
	FLOAT		m_fWeightDeltaVariation;	/* Variation of change in weight */

	D3DXCOLOR	m_cStandardStartColor;		/* Standard color of particle when emitted */
	D3DXCOLOR	m_cStartColorVariation;		/* Variation in start color */
	D3DXCOLOR	m_cStandardEndColor;		/* Standard color of particle when dying */
	D3DXCOLOR	m_cEndColorVariation;		/* Variation in end color */

	u_int32		m_iParticlesPerSecond;		/* Number of particles emitted per second */
};

class particle_skel : public scene  
{
public:

    particle_skel(char* opts = 0);

	int operator()();
    virtual ~particle_skel();
    virtual int _load();
	virtual int _restore();
    virtual int _render();

	int ch(char* x=0);

	int handle(char* type, char* ev);

protected:
		/* Vertex buffer to store point sprites */
	LPDIRECT3DVERTEXBUFFER9 m_pParticleBuffer;

	/* Texture that is used for the particle */
	//CTexture m_kParticleTexture;

	/* List of particles used by this system */
	Particle *m_kParticleList;

	/* Particle system properties */
	u_int32		m_iMaxParticles;	/* Max nr of particles */
	u_int32		m_iNumParticles;	/* Current nr of particles */
	D3DXVECTOR3	m_vOrigin;			/* Center of particle system */
	D3DXVECTOR3	m_vForce;			/* Force affecting system (gravity, wind, etc.) */
	real32		m_fAccumulatedTime; /* Time passed since last particle */
	
	/* Information about creating particles */
	ParticleInfo * m_pParticleInfo;
	ParticleInfo info;

	void InitializeParticle (u_int32 index);
	void  Update (real32 fTimePassed);
	//err32 Render (void);
	int   Emit	 (u_int32 iNrOfParticles);
	void  Reset (void);
	void  Move  (D3DXVECTOR3 vDirection);

	/* Maintenance */
	void SetOrigin (D3DXVECTOR3 vOrigin);
	void SetForce  (D3DXVECTOR3 vForce);

	D3DXVECTOR3 GetOrigin (void);
	D3DXVECTOR3 GetForce  (void);
	u_int32		GetNrOfParticles (void);

	int choice;


};
#endif 
