#include <hush/source.h>

#include <hush/event.h>
#include <hush/kit.h>
#include <hush/assert.h>


//#include <iostream.h>
#include <string.h>

//@class=event

class modevent : public event {
public:
modevent(const char* s = 0) : event(this), _name((char*)s) {}
modevent(event* e) : event(e), _name(0) { }
char* name() const  {  return _name?_name:"anevent"; }
private:
char* _name;
};

//@slide defaults

void* event::_last = 0;
void* event::_create = 0;
 
void set_simevent_fun(void* f) {
         event::_create = f;
         }

typedef event* (*createf)(int, int, int);
#define CREATE (event*) ((createf)_create)


#define TYPE event
#include "hush/sys/table.i"


//@class event

void* event::give_last() { return event::_last; }
void event::set_last(void* x) { event::_last = x; }

int event::exists() const { return _body && ( _body != this ); }

event::event(class kit* _tk, int _argc, char** _argv) : _del(0)  {
	_event = this;        // remember, event is its own handler!
	event::created++;
	event* e = event::lookup("tcl");
	if (e) { _body = e->create(_tk,_argc,_argv);
		_register(_body); // AE
		}
	}

event::event(const char* s) : _del(0) {
	event::created++;
	_event = this;        // remember, event is its own handler!
	_body = new modevent(s);
	_register(_body);
	}

event::event(const event& e) {
	event::created++;
	_event = this;        // remember, event is its own handler!
	_body = e._body;
	_del = e._del;
	}

event::event(event* e) : _body(e)   {
	event::created++;
	_event = this;        // remember, event is its own handler!
	_del = 0;
}

event::event(int qp,int sp,int k) : _del(1) {  /// for sim
	event::created++;
	_event = this;        // remember, event is its own handler!
	if (event::_create) 
	     _body = CREATE(qp,sp,k);
	     //if (_body) _register(_body);  // no need <-- _del
	else {
	     _body = 0; // new modevent("default");
	     //AE err() << "event::event: warning  sim not installed\n"; // << endl;
	     }
	}

event* event::create(kit* k,int argc, char** argv) {
	return exists()?_body->create(k,argc,argv):0;
	}

event* event::get() const { 
	return exists()?_body->get():0;
	}

event::~event() {
	event::deleted++;
	if (_body && _del) delete _body; _body = 0;
	}

int event::eventclass() const { 
	return exists()?_body->eventclass():0; // jrvosse
	}

//@slide Tcl/Tk

int event::argc() const {
	return exists()?_body->argc():0;
	}

int event::fd() const {
	return exists()?_body->fd():-1;
	}

int event::mask() const {
	return exists()?_body->mask():0;
	}

char** event::argv(int i ) const {
	return exists()?_body->argv(i):0;
	}

char* event::arg( int i ) const {
	return exists()?_body->arg(i):0;
	}

char* event::args(int n, int q) const {
	int _argc = argc(); 
	if (!_argc) return "";
	char** _argv = argv(0);
	string s(_argc-n,_argv+n);
	char* p = (char*) s.flatten(q);
	return p;
	}

//@ ask & tell

char* event::ask() {
	return exists()?_body->ask():0; 
	}

void event::tell(const char* msg) {
	if (exists()) _body->tell(msg);
	}

//@slide dispatching


int event::operator()() {
	return handler::operator()();
	}

//@slide general

int event::type() const { return exists()?_body->type():0; } /*press*/
char* event::name() const { return exists()?_body->name():"anonymous"; }
void event::trace() const { if (exists()) _body->trace(); }

int event::no() const { return exists()?_body->no():0; }
char* event::id() const { return exists()?_body->id():"anonymous"; }

//@slide x-event

int event::x() const { return exists()?_body->x():0; }
int event::y() const { return exists()?_body->y():0; } 
int event::x_root() const { return exists()?_body->x_root():0; }
int event::y_root() const { return exists()?_body->y_root():0; } 
int event::button(int i) const { return exists()?_body->button(i):0; }
int event::buttonup(int i) const { return exists()?_body->buttonup(i):0; }
int event::keycode() const { return exists()?_body->keycode():0; }
int event::buttonevent(int i) const { return exists()?_body->buttonevent(i):0; }
int event::keyevent() const { return exists()?_body->keyevent():0; }
int event::motion() const { return exists()?_body->motion():0; }
void event::motion( event& ) { }
void* event::rawevent() { return exists()?_body->rawevent():0; }

//@slide sim-event

int event::report(histogram* hi,double interv) { 
	return exists()?_body->report(hi,interv):0;
	} 
void  event::stamp() { if (exists()) _body->stamp(); }

double  event::timespent() { return exists()?_body->timespent():0; }

char*  event::reporttitle() { return exists()?_body->reporttitle():0; }
double  event::time() const { return exists()?_body->time():0; }
void  event::time(double t) { if (exists()) _body->time(t); }
int  event::schedulingpriority() { return exists()?_body->schedulingpriority():0; }
void  event::schedulingpriority(int p) { if (exists()) _body->schedulingpriority(p); }
int  event::queuingpriority() { return exists()?_body->queuingpriority():0; }
void  event::queuingpriority(int p) { if (exists()) _body->queuingpriority(p); }
double  event::interval() { return exists()?_body->interval():0; }
void  event::interval(double i) { if (exists()) _body->interval(i); }
int  event::state() { return exists()?_body->state():0; }
void  event::state(int s) { if (exists()) _body->state(s); }
int  event::killflag() { return exists()?_body->killflag():0; }
event*  event::leftpointer() { return exists()?_body->leftpointer():0; }
void  event::leftpointer(event* l) { if (exists()) _body->leftpointer(l); } 
event*  event::middlepointer() { return exists()?_body->middlepointer():0; }
void  event::middlepointer(event* l) { if (exists()) _body->middlepointer(l); }
event*  event::rightpointer() { return exists()?_body->rightpointer():0; }
void  event::rightpointer(event* l) { if (exists()) _body->rightpointer(l); }
event*  event::parentpointer() { return exists()?_body->parentpointer():0; }
void  event::parentpointer(event* l) { if (exists()) _body->parentpointer(l); }
double  event::waitingreporttime() { return exists()?_body->waitingreporttime():0; }
void  event::waitingreporttime(double w) { if (exists()) _body->waitingreporttime(w); }
double  event::eventreporttime() { return exists()?_body->eventreporttime():0; }
int  event::_active() { return exists()?_body->_active():0; }
int  event::pending() { return exists()?_body->pending():0; }
int  event::conditional() { return exists()?_body->conditional():0; }
int  event::closed() { return exists()?_body->closed():0; }
int  event::passive() { return exists()?_body->passive():0; }
int  event::queued() { return exists()?_body->queued():0; }


//@  xml-like

char* event::_tag() const { return exists()?_body->_tag():0; }
char* event::_attributes() const { return exists()?_body->_attributes():0; }
char* event::_data() const { return exists()?_body->_data():0; }

//@ logic

long event::eval(const char* cmd) { return tk?tk->eval(cmd):0; }
char* event::evaluate(const char* cmd) { return tk?tk->evaluate(cmd):0; }

char* event::result(long id) { return tk?tk->result(id):0; }
void event::result(const char* s) {
	tell(s);
	}

//@ native logic events

char* event::_box() { return exists()?_body->_box():option("box"); }   
char* event::_process() { return exists()?_body->_process():option("process"); }
char* event::_thread() { return exists()?_body->_thread():option("thread"); }

char* event::_instream() { return exists()?_body->_instream():option("instream"); }
char* event::_outstream() { return exists()?_body->_outstream():option("outstream"); }

char* event::_class() { return exists()?_body->_class():option("class"); }
char* event::_object() { return exists()?_body->_object():option("object"); }
char* event::_objectid() { return exists()?_body->_objectid():option("objectid"); }
char* event::_reference() { return exists()?_body->_reference():option("reference"); }
char* event::_pointer() { return exists()?_body->_pointer():option("pointer"); }
char* event::_ior() { return exists()?_body->_ior():option("ior"); }
char* event::_method() { return exists()?_body->_method():option("method"); }

char* event::_variable() { return exists()?_body->_variable():option("variable"); }
char* event::_constraint() { return exists()?_body->_constraint():option("constraint"); }
char* event::_relation() { return exists()?_body->_relation():option("relation"); }
//</> 
