//@ kit.c -- I like kits, I make them myself
// See kit(4), issues(4), <hush/kit.h>.

#include <hush/source.h>

#include <hush/kit.h>

#include <hush/session.h>
#include <hush/event.h>
#include <hush/sys/bind.h>

#include <hush/assert.h>
#include <hush/agent/loader.h>

#include <hush/api/dot.h>
#include <hush/api/factory.h>

#include <hush/ml/parser.h>
#include <hush/ml/scanner.h>
#include <hush/ml/format.h>

//#include <iostream.h>
#include <stdio.h>

#ifndef menu
#define menu dvmenu
#endif

kit* thekit() { return 0; /* kit::_default; */ }

kit* kit::_default = 0;
void* kit::_create = 0;

typedef kit* (*createf)(char*);
#define CREATE (kit*) ((createf)kit::_create)


void kit::set_default(kit* x) { _default = x; }
kit* kit::get_default() { return _default; }


static
kit* a_new_kit( const char* mode ) {
        kit* tk =  kit::_default;
	if (!tk) tk = kit::lookup(mode);
        if (!tk) tk = kit::lookup("tk");
        if (!tk) tk = kit::lookup("tcl");
        if (!tk) tk = kit::lookup("hush");
        if (!tk) tk = kit::lookup("global");
	else tk = kit::_default;
        return tk;
        }

#define TYPE kit
//#define TABLE handlers
#include "hush/sys/table.i"

kit::kit(kit*) : tk(0) { kit::created++; _mode=0;}
kit::kit(const char* x) { 
	 _mode=0;
         if (!kit::_create) kit::_create = (void*) a_new_kit;
	 tk = CREATE((char*)x); kit::created++;
	 }

kit::~kit() { 
	kit::deleted++;
	}

// loader

int kit::load(const char* lib) {
	string s = lib;
	s.split(":");
        string* p = 0;
	iter<string>& it = s;
	loader l;
	while ( (p = it()) ) {
		l.load(*p);
		}
	return 1;
	}

kit* kit::check(const char* kind) {
	kit* _tk = 0;
	string s = kind;
	s.split(":");
        string* p = 0;
	iter<string>& it = s;
	while ( !_tk && (p = it()) ) {
	        char* k = (char*) *p;
		load(k);
		_tk = kit::lookup(k);
		}
	return _tk;
	}

// query / logic

term* kit::query(const char* pred,int md) {
kit* _tk = check("pl:logic");
int _md = (md)?md:_mode;
if (_tk) {
	return _tk->query(pred,_md);
	}
else return 0;
}

int kit::consult(const char* pred) {
kit* _tk = check("pl:logic");
if (_tk) {
	return _tk->consult(pred);
	}
else return 0;
}


// evaluation -- tcl/tk (basically)

long kit::eval(const char* cmd) { 
	int result = tk?tk->eval(cmd):0;
	return result;
	}

char* kit::evaluate(const char* cmd) {
	if (tk) {
		int res = tk->eval(cmd);
		return result(res);
		}
	return 0;
	}

char* kit::result(long id) { return tk?tk->result(id):0; }
void kit::result(const char* s) { if (tk) tk->result(s); }

void kit::_assert(const char* s, const char* opt) { if (tk) tk->_assert(s,opt); }
void kit::_retract(const char* s, const char* opt) { if (tk) tk->_retract(s,opt); }

int kit::source(const char* f) { return tk?tk->source(f):0; }

int kit::error() { return tk?tk->error():0; }
void kit::error(const char* msg) { if (tk) tk->error(msg); }

//void kit::after(int n, const char* cmd) { if (tk) tk->after(n,cmd); }
//void kit::after(int n, void* f, void* c) { if (tk) tk->after(n,f,c); }

void kit::after(int n, handler* h) { if (tk) tk->after(n,h); }
void* kit::token() const { return tk?tk->token():0; }

char* kit::send(const char* it, const char* cmd) { 
	return tk?tk->send(it,cmd):0;
	}

char* kit::selection(const char* options) const {
	return tk?tk->selection(options):0;
	}

void kit::update(const char* options) { if (tk) tk->update(options); }


int kit::safe() const {
	return tk?tk->safe():0;
	}

kit* kit::slave( char* options ) {
	return tk?tk->slave(options):0;
	}


 
void kit::unbind(int fd) {
	if (tk) tk->unbind(fd);
	}

void kit::unbind(char* name, char* newname) {
	if (tk) tk->unbind(name,newname);
	}


binding* kit::bind(const char* name, handler* h) {
	//err() << "kit::binding " << name << "\n";
	return tk?tk->bind(name,h):0;
	}


binding* kit::bind(handler* h) {
	return tk?tk->bind(h):0;
	}

binding* kit::bind(int fd, handler* h, int mask) { // jrvosse
	return tk?tk->bind(fd,h, mask):0;
	}

void kit::option(const char* k, const char* v) {
	if (tk) tk->option(k,v);
	}

char* kit::option(const char* k) {
	return tk?tk->option(k):0;
	}


void kit::quit() {
	session::_quit = 1;
	if (tk) tk->quit();
	}

void kit::trace(int i) { session::_trace = i; }
void kit::notrace() { session::_trace = 0; }
int kit::tracelevel() const { return session::_trace; }

widget* kit::root() const { return tk?tk->root():0;}

widget* kit::pack(widget* w, const char* opts) {
	kit* _tk = kit::lookup("tk");
	if (_tk) return _tk->pack(w,opts);
	else return tk?tk->pack(w,opts):0; 
	}

widget* kit::pack(const char* w, const char* opts) {
	kit* _tk = kit::lookup("tk");
	if (_tk) return _tk->pack(w,opts);
	if ( tracelevel() > 5 )
		;//err() << "kit::pack (char*)" << endl; 
	return tk?tk->pack(w,opts):0;
	}

int kit::mode() const { return tk?tk->mode():_mode; }
int kit::mode(int m) { 
	        if (tk)  return tk->mode(m);
		int tmp = _mode; _mode = m; return tmp;
		}
	
void* kit::interp() const { return tk?tk->interp():0; }
void* kit::tkwin(const char* path) const { return tk?tk->tkwin(path):0; }
void* kit::display() const { return tk?tk->display():0; }

void kit::put(event* p,int fl) { if (tk) tk->put(p,fl); }

event* kit::get() const { 
	if ( tracelevel() > 5 )
	        ;//err() << "kit is asked for an event" << endl;
	return tk?tk->get():0; //new class event("dummy");
	}

//// Experimental -- role switching

void kit::font(char* s) { if (tk) tk->font(s); }

int kit::alias(char* s, handler* h) {
	return tk?tk->alias(s,h):0;
	}

kit* kit::become(char* s, kit* k) {
	if (s && k); //Wall
	;//AE err() << "NYET : kit::become " << s << " " << k << endl;
	return tk;
	}

void kit::restore(kit* k) {
	if (k); //Wall
	//err() << "NYET : kit::restore " << k << endl;
	}

// special kit and parser factory

kit* kit::_kit(const char* k) {
	kit* x = kit::lookup(k);
	if (x) return x;
	else {
          this->load(k);
	  x = kit::lookup(k);
	  return x;
	  }
	}

::parser* kit::parser(const char* k) {
	::parser* x = ::parser::lookup(k);
	if (x) return x;
	else {
	  char buf[1024];
	  sprintf(buf,"%sgrammar",k);
          this->load(buf);
	  x = ::parser::lookup(k);
	  return x;
	  }
	}

::scanner* kit::scanner(const char* k) {
	::scanner* x = ::scanner::lookup(k);
	if (x) return x;
	else {
	  char buf[1024];
	  sprintf(buf,"%sgrammar",k);
          this->load(buf);
	  x = ::scanner::lookup(k);
	  return x;
	  }
	}

::formatter* kit::formatter(const char* k) {
	::formatter* x = ::formatter::lookup(k);
	if (x) return x;
	else {
	  char buf[1024];
	  sprintf(buf,"%sgrammar",k);
          this->load(buf);
	  x = ::formatter::lookup(k);
	  return x;
	  }
	}

// dot + factory

::dot* kit::dot(const char* k) {
	::dot* tmp = _dot;
	if (!k && _dot) return _dot;
	else if (k) {
		// get a dot from dot;
		return tmp;
		}
	else if (!_dot) {
		_dot = new ::dot();
		_register(_dot);
		return _dot;
		}
	return _dot;
	}

::factory* kit::factory(const char* k) {
	::factory* tmp = _factory;
	if (!k && _factory) return _factory;
	else if (k) {
		// get a factory from factory;
		return tmp;
		}
	else if (!_factory) {
		_factory = new ::factory();
		_register(_factory);
		return _factory;
		}
	return _factory;
	}


#define fac(button) \
::button* kit::button(const char* k) { return dot()->button(k); } \
::button* kit::button(widget* r, const char* p, const char* opts) { \
	return factory()->button(r,p,opts); \
	}

fac(button)
fac(checkbutton)
fac(radiobutton)

fac(menu)
fac(menubutton)
fac(menubar)

fac(message)
fac(entry)
fac(listbox)

fac(frame)
fac(toplevel)

fac(scrollbar)
fac(label)

fac(canvas)
fac(dvtext)

fac(filechooser)
fac(browser)


