#include <hush/source.h>

// abstract widget class

#include <hush/widget.h>
#include <hush/item.h>
#include <hush/sys/bind.h>

#include <hush/assert.h>

#include <string.h>
#include <stdio.h>

typedef class handler ancestor;

int widget::widgetno = 0;

#define TYPE widget
#define DESTROY _destroy
#include "hush/sys/table.i"

//// The widget 'dispatch logic' for inner and outer contexts

widget* widget::_default = 0;

void widget::set_default(widget* x) { _default = x; }
widget* widget::get_default() { return _default; }   

int widget::invariant() const { // _body is private, _context() is from widget
	return handler::invariant() && 1 ;
	  //  (_body() != this) && !(_body() && _context())
	}

//// To check whether there is an inner or outer context

widget* widget::_body() const { return (widget*) _bdy; }  // private

int widget::_exists() const { return _this() != this; }

widget* widget::_this() const {        // private
	require( widget::invariant() );
	if (exists()) return self();
	else if (_body()) return  _body();
	else if (_context()) return  _context();
	else return (widget*) this;
	}


// General self dispatch -- see widgets/hush/dispatch.i

//int widget::exists() const { return self() != this; }


widget* widget::_context() const { 
	widget* w = (widget*) handler::_context(); 
	return (w == this)?0:w;
	}

widget* widget::_self() const {  //AE: obscure
	return (widget*) ancestor::_self();
	}

int widget::exists() const   { 
	widget* x = _self();
	if (x && x != this) return 1;
	else return 0;
	}

widget* widget::self() const {  
	if (exists()) return _self()->self();
	else return (widget*) this;
	}

//// General redirect for compound widgets

void widget::redirect(widget* w) {
	handler* result = 0;
	result = ancestor::_redirect(w); // from handler
	//if (!_aliased)
		alias(this);
}


//// Widget constructors

widget::widget( ) { 
	buf = 0; _path = 0; _bdy = 0;
	widget::created++;
	}

widget::widget(const char* p, const char* opts, widget* ctx) {
	buf = 0; _path = 0; _bdy  = 0;
	widget::created++;
	if (ctx && ctx != this) _reparent(ctx);
	if (!ctx || ctx == this) {
				setpath(p);
				_debug("widget",p);
				}
	if ( (!ctx || ctx == this) && widget::_default) {
		_bdy = widget::_default->_create(p,opts,this);
		_bdy->_reparent( this );
		_register(_bdy);
		}
	if (_bdy) tk = kit::lookup(_bdy->category());
	else if (_context()) tk = kit::lookup(_context()->category());
	//AE if (!tk) tk = kit::_default;
	require( tk && widget::invariant() );
	buf = new char[BUFSIZ];
	}

widget::widget(const widget* w, const char* p, const char* opts,
				widget* ctx) {
	buf = 0; _path = 0; _bdy = 0;
	widget::created++;
	if (ctx && ctx != this) _reparent(ctx);
	if (!ctx || ctx == this) setpath(w,p);
	if ( (!ctx || ctx == this) && widget::_default) {
		if (w)
			_bdy = widget::_default->_create(w,p,opts,this);
		else
			_bdy = widget::_default->_create(p,opts,this);
		_bdy->_reparent( this );
		}
	if (_bdy) tk = kit::lookup(_bdy->category());
	else if (_context()) tk = kit::lookup(_context()->category());
//AE	if (!tk) tk = kit::_default;
	require( tk && invariant() );
	buf = new char[BUFSIZ];
	if (w) w->_register(this); /// automatic registration
	}

widget* widget::_create(const char* p, const char* opts, widget* ctx) {
	if (_exists()) return _this()->_create(p,opts,ctx);
	else return new widget(p,opts,ctx);
	}

widget* widget::_create(const widget* w, const char* p,
				       const char* opts, widget* ctx) {
	if (_this() != this) return _this()->_create(w,p,opts,ctx);
	else return new widget(w,p,opts,ctx);
	}


widget::~widget() {
	widget::deleted++;
	if (buf) delete[] buf; buf = 0;
	if (_path) delete[] _path; /// gives a core dump???
}

char* widget::info() const { 
	//if (_context()) return _context()->info(); 
	if (_body()) return _body()->info();
	else return type();
	}


char* widget::type() const { 
	if (_context()) return _context()->type(); 
	return "widget";
	}

char* widget::name() const { 
	if (_context()) return _context()->name(); 
	char* s = ( handler::name() ? handler::name() : path() ); 
	return s;
	}

void widget::title(const char* s) {
	if (_exists()) _this()->title(s);
	else _error("title",path());
	}

char* widget::title() const {
	if (_exists()) return _this()->title();
	else return (char*) _error("title",path());
	}
	

void widget::alias(handler* w) {    // caller default 0 AE XXXX
	if (_context()) _context()->alias(w); 
	else if (_body()) _body()->alias(w);
	else _error("alias", path());
	}

void widget::image(const char* s) { 
	if (_exists())  _this()->image(s);
	else _error("image",path());
	}

void widget::cursor(const char* s) { 
	if (_exists())  _this()->cursor(s);
	else _error("cursor",path());
	}


int widget::width()  const {
	if (_exists()) return _this()->width();
	else return (int) _error("width",path());
	}

int widget::height()  const {
	if (_exists()) return _this()->height();
	else return (int) _error("height",path());
	}


char* widget::path() const {
	if (_context()) return _context()->path();
	else return (char*)  _path;
	}


long widget::eval(const char* s) {
	if (_exists()) return _this()->eval(s);
	else return (long) _error("eval",s);
	}


char* widget::result(long id) {
	if (_exists()) return _this()->result(id);
	return (char*) _error("result",path());
	}

char* widget::evaluate(const char* s) {
	if (_exists()) return _this()->evaluate(s);
	else {
		long res = eval(s); return result(res);
	}
}


void widget::destroy() {
	if (_exists()) _this()->destroy();
	/*
	widget* w = _body();
	if (w) w->destroy();
	*/
	else _error("destroy",path());
	}

void widget::xscroll(const widget* w) {
	if (_exists()) _this()->xscroll(w);
	else _error("xscroll");
	}
	

void widget::yscroll(const widget* w) {
	if (_exists()) _this()->yscroll(w);
	else _error("yscroll");
	}

void widget::configure(const char* s) { 
	if (_exists()) _this()->configure(s);
	else _error("configure",path());
	}

void widget::geometry(int x, int y) {
	if (_exists()) _this()->geometry(x,y);
	else _error("geometry");
	}

widget* widget::pack(widget* w, const char* opts) {
	if (_exists()) return _this()->pack(w,opts);
	else _error("pack",path());
	return this;
	}

void widget::place(const char* opts) {
	if (_exists()) _this()->place(opts);
	else _error("place", path());
	}

int widget::bind(const char *s, dvbinding* ac, const char* args) {
	return _this()->direct(s,ac,args);
	}

int widget::bind(dvbinding* ac, const char* args) {
	_this()->install(ac,args);
	return 0;
	}

int widget::bind(const char *s, class handler* h, const char* args) {
	if (_body()) return _body()->bind(s,h,args);
	else {
		binding* ac = tk->bind(h);
		return _this()->direct(s,ac,args);
	}
}


int widget::bind(class handler* h, const char* args) {
	if (_body()) return _body()->bind(h,args);
	else {
		_debug("bind",name());
		if (tk) { 
			binding* ac = tk->bind(h);
			_this()->install(ac,args);
			}
		return 0;
	}
}

widget* widget::pack(const char* opts) {
	//err() << " widget::pack " << path() << endl;
	if (_body()) return _body()->pack(opts);
	else _error("pack", path());
	return this;
	}


void widget::source(const char* kind, const char* args, const char* opts) {
	if (_exists()) _this()->source(kind,args,opts);
	else _error("source",path());
	}

void widget::target(const char* kind, dvbinding* ac,
			const char* args, const char* opts) {
	if (_exists()) _this()->target(kind,ac,args,opts);
	else _error("target");
	}


void widget::focus(const char* opts) {
	if (_exists()) _this()->focus(opts);
	else _error("source");
	}

void widget::grab(const char* opts) {
	if (_exists()) _this()->grab(opts);
	else _error("grab", path());
	}


void widget::wait() {
	if (_exists()) _this()->wait();
	else _error("wait",path());
	}


void widget::map() {
	if (_exists()) _this()->map(); 
	else _error("map");
	}

void widget::unmap() {
	if (_exists()) _this()->unmap(); 
	else _error("unmap");
	}

void* widget::tkwin() const {
	if (_exists()) return _this()->tkwin();
	else return _error("tkwin");
	}

void widget::font(char* s) {
	if (_exists()) _this()->font(s);
	else _error("font",path());
	}

void widget::prefix(const widget* p) {
	if (_exists()) _this()->prefix(p);
	else setpath(p,path());
	}

char* widget::thepath() const { return self()->path(); }

void widget::install(binding* ac, const char* args) {
	if (_exists()) _this()->install(ac,args);
	else _error("install",path());
	}

int widget::direct(const char *s, binding* ac, const char* args) {
        _debug("direct",s);
	if (_exists()) return _this()->direct(s,ac,args); 
	else return (int) _error("direct",path());
	}

void widget::setpath( const char* s) {
	if (0 && _context()) _context()->setpath(s);
	else {
		char* tmp = new char[strlen(s)+2];
		if (s[0] != '.') sprintf(tmp,".%s",s);
		else sprintf(tmp,"%s",s);
		_path = tmp;
	}
}

void widget::setpath( const widget* w, const char* s) { 
	if (0 && _context()) _context()->setpath(w,s);
	else {
		int pad = 10; // was 4
		char* wp = w?w->path():""; // AE ? core dump
		int k = wp?strlen(wp):0;
		char* tmp = new char[k + strlen(s) + pad];
		if (s[0] != '.' && s[0] != '-' && s[0] != '_')
				sprintf(tmp,"%s.%s",wp?wp:"",s);
		else sprintf(tmp,"%s%s",wp?wp:"",s);
		char* result = tmp;
		if( tmp[0]=='.' && tmp[1]=='.' ) {
			result = new char[ strlen(tmp) ];
			strcpy(result,tmp+1);
			delete tmp;
			}
		_path = result;;
		//_register((string*)_path); delete tmp; // ERRORS
	}
}


void widget::init(const char* opts) {
	if (_context()) _context()->init(opts);
	else if (_body()) _body()->init(opts);
	else _error("init",path());
	}

int widget::operator()() { // AE
	_debug("()",path());
	if (_context()) {
		_debug("switching () context",path());
		event* e = _context()->dispatch(_event);
		return e?1:0;
		}
	else return ancestor::operator()();
	}

char* widget::cget(const char* opt) const {
	//if (_self()) return _self()->cget(opt); 
	if (_exists()) return _this()->cget(opt);
	else return (char*) _error("cget",path());
	}

char* widget::bindtags() const {
      if (_exists()) return _this()->bindtags();
      else return (char*) _error("bindtags",path());
      }
 
void widget::bindtags(const char* bindorder) {
      if (_exists()) _this()->bindtags(bindorder);
      else _error("bindtags",path());
      }
