#include <hush/source.h>
#include <hush/midi/file.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

//#include <hush/io.h>
#include <hush/iostream.h>

midi_file::midi_file() {
	Mf_nomerge = 0;
	Mf_currtime = 0L;
	Mf_toberead = 0L;
	Mf_numbyteswritten = 0L;
	fp_in = stdin; fp_out = stdout;
	_division = 0;
	_tempo = 0; _file = 0; 
	_track = 0; _tracks = 0; _in_seconds = 0;
	_handler = this;
	_in_track = 0; _channel = 0; _trackno = 0;
	}

FILE* midi_file::in_file() { return fp_in?fp_in:stdin; }
FILE* midi_file::out_file() { return fp_out?fp_out:stdout; }

int midi_file::division() { return _division; }
int midi_file::track() { return _track; }
int midi_file::tracks() { return _tracks; }
int midi_file::tempo() { return _tempo; }
char* midi_file::file() { return _file; }
long midi_file::time() { return Mf_currtime; }
double midi_file::seconds() { return mf_ticks2sec(Mf_currtime,_division,_tempo); }

int midi_file::in_seconds() { return _in_seconds; }


int midi_file::write(const char* f, int format, int trcks, int division) {
	FILE* fp = 0;
	if (f)
		fp = fopen(f,"w");
	fp_out = fp;
	mfwrite(format,trcks,division,fp);
	return 1;
	}

int midi_file::open(const char* f, const char* mode) {
	FILE* fp = 0;
	if (f)
		fp = fopen(f,mode);
	if (fp && !strcmp(mode,"w")) fp_out = fp;
	else if ( fp && !strcmp(mode,"r")) fp_in = fp;
	return fp?1:0;
	}

int midi_file::close() {
	//eputc('\n');
	//fclose(fp_out);
	return 0;
	}

/* public stuff */


void midi_file::mfread( FILE* fp)	 	/* The only non-static function in this file. */
{
	fp_in = fp;
	readheader();
	while ( readtrack() )
		;
}

/* for backward compatibility with the original lib */
void midi_file::midifile(const char* f, const char* out)
{
    fp_in = stdin; fp_out = stdout;
    if (f) _file = (char*) f; else _file = "unknown";
    FILE* fp = 0;
    if (f) fp = fopen(f,"r");
    if (f && fp) fp_in = fp;
    if (out) fp = fopen(out,"w");
    if (out && fp) fp_out = fp;
    //cerr << "reading " << (void*) fp_in << " " <<  (void*) fp_out << endl;
    mfread(fp_in);
    if (f) fclose(fp_in);
    if (out) fclose(fp_out);
}

int midi_file::readmt(char* s)		/* read through the "MThd" or "MTrk" header string */
{
	int n = 0;
	char *p = s;
	int c;

	while ( n++<4 && (c=_handler->midi_getc()) != EOF ) {
		if ( c != *p++ ) {
			char buff[32];
			(void) strcpy(buff,"expecting ");
			(void) strcat(buff,s);
			mferror(buff);
		}
	}
	return(c);
}

int midi_file::egetc() // read a single character and abort on EOF
{
	int c = _handler->midi_getc();

	if ( c == EOF )
		mferror("premature EOF");
	Mf_toberead--;
	return(c);
}

int midi_file::readheader()		/* read a header chunk */
{
	int format, ntrks, division;

	if ( readmt("MThd") == EOF )
		return 0;

	Mf_toberead = read32bit();
	format = read16bit();
	_tracks = ntrks = read16bit();
	_division = division = read16bit();

		_handler->midi_header(format,ntrks,division);

	/* flush any extra stuff, in case the length of header is not 6 */
	while ( Mf_toberead > 0 )
		(void) egetc();
return 1;
}

int midi_file::readtrack()		 /* read a track chunk */
{
	/* This array is indexed by the high half of a status byte.  It's */
	/* value is either the number of bytes needed (1 or 2) for a channel */
	/* message, or 0 (meaning it's not  a channel message). */
	static int chantype[] = {
		0, 0, 0, 0, 0, 0, 0, 0,		/* 0x00 through 0x70 */
		2, 2, 2, 2, 1, 1, 2, 0		/* 0x80 through 0xf0 */
	};
	long lookfor;
	int c, c1, type;
	int sysexcontinue = 0;	/* 1 if last message was an unfinished sysex */
	int running = 0;	/* 1 when running status used */
	int status = 0;		/* status value (e.g. 0x90==note-on) */
	int needed;

	if ( readmt("MTrk") == EOF )
		return(0);

	Mf_toberead = read32bit();
	Mf_currtime = 0;

		_track++;
		_handler->midi_trackstart();

	while ( Mf_toberead > 0 ) {

		Mf_currtime += readvarinum();	/* delta time */

		c = egetc();

		if ( sysexcontinue && c != 0xf7 )
			mferror("didn't find expected continuation of a sysex");

		if ( (c & 0x80) == 0 ) {	 /* running status? */
			if ( status == 0 )
				mferror("unexpected running status");
			running = 1;
		}
		else {
			status = c;
			running = 0;
		}

		needed = chantype[ (status>>4) & 0xf ];

		if ( needed ) {		/* ie. is it a channel message? */

			if ( running )
				c1 = c;
			else
				c1 = egetc();
			chanmessage( status, c1, (needed>1) ? egetc() : 0 );
			continue;;
		}

		switch ( c ) {

		case 0xff:			/* meta event */

			type = egetc();
			lookfor = Mf_toberead - readvarinum();
			msginit();

			while ( Mf_toberead > lookfor )
				msgadd(egetc());

			metaevent(type);
			break;

		case 0xf0:		/* start of system exclusive */

			lookfor = Mf_toberead - readvarinum();
			msginit();
			msgadd(0xf0);

			while ( Mf_toberead > lookfor )
				msgadd(c=egetc());

			if ( c==0xf7 || Mf_nomerge==0 )
				sysex();
			else
				sysexcontinue = 1;  /* merge into next msg */
			break;

		case 0xf7:	/* sysex continuation or arbitrary stuff */

			lookfor = Mf_toberead - readvarinum();

			if ( ! sysexcontinue )
				msginit();

			while ( Mf_toberead > lookfor )
				msgadd(c=egetc());

			if ( ! sysexcontinue ) {
					_handler->midi_arbitrary(msgleng(),msg());
			}
			else if ( c == 0xf7 ) {
				sysex();
				sysexcontinue = 0;
			}
			break;
		default:
			badbyte(c);
			break;
		}
	}
		_handler->midi_trackend();
	return(1);
}

void midi_file::badbyte(int c)
{
	char buff[32];

	(void) sprintf(buff,"unexpected byte: 0x%02x",c);
	mferror(buff);
}

void midi_file::metaevent(int type)
{
	int leng = msgleng();
	char *m = msg();

	switch  ( type ) {
	case 0x00:
			_handler->midi_seqnum(to16bit(m[0],m[1]));
		break;
	case 0x01:	/* Text event */
	case 0x02:	/* Copyright notice */
	case 0x03:	/* Sequence/Track name */
	case 0x04:	/* Instrument name */
	case 0x05:	/* Lyric */
	case 0x06:	/* Marker */
	case 0x07:	/* Cue point */
	case 0x08:
	case 0x09:
	case 0x0a:
	case 0x0b:
	case 0x0c:
	case 0x0d:
	case 0x0e:
	case 0x0f:
		/* These are all text events */
			_handler->midi_text(type,leng,m);
		break;
	case 0x2f:	/* End of Track */
			_handler->midi_eot();
		break;
	case 0x51:	/* Set tempo */
			_tempo = to32bit(0,m[0],m[1],m[2]);
			_handler->midi_tempo(_tempo);
		break;
	case 0x54:
			_handler->midi_smpte(m[0],m[1],m[2],m[3],m[4]);
		break;
	case 0x58:
			_handler->midi_timesig(m[0],m[1],m[2],m[3]);
		break;
	case 0x59:
			_handler->midi_keysig(m[0],m[1]);
		break;
	case 0x7f:
			_handler->midi_seqspecific(type,leng,m);
		break;
	default:
			_handler->midi_metamisc(type,leng,m);
	}
}

void midi_file::sysex()
{
		_handler->midi_sysex(msgleng(),msg());
}

void midi_file::chanmessage(int status, int c1, int c2)
{
	int chan = status & 0xf;

	switch ( status & 0xf0 ) {
	case 0x80:
			_handler->midi_noteoff(chan,c1,c2);
		break;
	case 0x90:
			_handler->midi_noteon(chan,c1,c2);
		break;
	case 0xa0:
			_handler->midi_pressure(chan,c1,c2);
		break;
	case 0xb0:
			_handler->midi_parameter(chan,c1,c2);
		break;
	case 0xe0:
			_handler->midi_pitchbend(chan,c1,c2);
		break;
	case 0xc0:
			_handler->midi_program(chan,c1); //AE
		break;
	case 0xd0:
			_handler->midi_chanpressure(chan,c1); //AE
		break;
	}
}

/* readvarinum - read a varying-length number, and return the */
/* number of characters it took. */

long midi_file::readvarinum()
{
	long value;
	int c;

	c = egetc();
	value = c;
	if ( c & 0x80 ) {
		value &= 0x7f;
		do {
			c = egetc();
			value = (value << 7) + (c & 0x7f);
		} while (c & 0x80);
	}
	return (value);
}

long midi_file::to32bit(int c1, int c2, int c3, int c4)
{
	long value = 0L;

	value = (c1 & 0xff);
	value = (value<<8) + (c2 & 0xff);
	value = (value<<8) + (c3 & 0xff);
	value = (value<<8) + (c4 & 0xff);
	return (value);
}

int midi_file::to16bit(int c1, int c2)
{
	return ((c1 & 0xff ) << 8) + (c2 & 0xff);
}

long midi_file::read32bit()
{
	int c1, c2, c3, c4;

	c1 = egetc();
	c2 = egetc();
	c3 = egetc();
	c4 = egetc();
	return to32bit(c1,c2,c3,c4);
}

int midi_file::read16bit()
{
	int c1, c2;
	c1 = egetc();
	c2 = egetc();
	return to16bit(c1,c2);
}

void midi_file::mferror(char *s)
{
	_handler->midi_error(s);
	//exit(1);
}

/* The code below allows collection of a system exclusive message of */
/* arbitrary length.  The Msgbuff is expanded as necessary.  The only */
/* visible data/routines are msginit(), msgadd(), msg(), msgleng(). */

#define MSGINCREMENT 128
static char *Msgbuff = NULL;	/* message buffer */
static int Msgsize = 0;		/* Size of currently allocated Msg */
static int Msgindex = 0;	/* index of next available location in Msg */

void midi_file::msginit()
{
	Msgindex = 0;
}

char * midi_file::msg()
{
	return(Msgbuff);
}

int midi_file::msgleng()
{
	return(Msgindex);
}

void midi_file::msgadd(int c)
{
	/* If necessary, allocate larger message buffer. */
	if ( Msgindex >= Msgsize )
		biggermsg();
	Msgbuff[Msgindex++] = c;
}

void midi_file::biggermsg()
{
/* 	char *malloc(); */
	char *newmess;
	char *oldmess = Msgbuff;
	int oldleng = Msgsize;

	Msgsize += MSGINCREMENT;
	newmess = (char *) malloc( (unsigned)(sizeof(char)*Msgsize) );

	if(newmess == NULL)
		mferror("malloc error!");
		
	/* copy old message into larger new one */
	if ( oldmess != NULL ) {
		register char *p = newmess;
		register char *q = oldmess;
		register char *endq = &oldmess[oldleng];

		for ( ; q!=endq ; p++,q++ )
			*p = *q;
		free(oldmess);
	}
	Msgbuff = newmess;
}

/*
 * mfwrite() - The only fuction you'll need to call to write out
 *             a midi file.
 *
 * format      0 - Single multi-channel track
 *             1 - Multiple simultaneous tracks
 *             2 - One or more sequentially independent
 *                 single track patterns                
 * ntracks     The number of tracks in the file.
 * division    This is kind of tricky, it can represent two
 *             things, depending on whether it is positive or negative
 *             (bit 15 set or not).  If  bit  15  of division  is zero,
 *             bits 14 through 0 represent the number of delta-time
 *             "ticks" which make up a quarter note.  If bit  15 of
 *             division  is  a one, delta-times in a file correspond to
 *             subdivisions of a second similiar to  SMPTE  and  MIDI
 *             time code.  In  this format bits 14 through 8 contain
 *             one of four values - 24, -25, -29, or -30,
 *             corresponding  to  the  four standard  SMPTE and MIDI
 *             time code frame per second formats, where  -29
 *             represents  30  drop  frame.   The  second  byte
 *             consisting  of  bits 7 through 0 corresponds the the
 *             resolution within a frame.  Refer the Standard MIDI
 *             Files 1.0 spec for more details.
 * fp          This should be the open file pointer to the file you
 *             want to write.  It will have be a global in order
 *             to work with Mf_putc.  
 */ 
void midi_file::mfwrite(int format, int ntracks, int division, FILE* fp) 
{
    fp_out = fp;
    int i;// void mf_write_track_chunk(), mf_write_header_chunk();


    /* every MIDI file starts with a header */
    mf_write_header_chunk(format,ntracks,division);

    /* In format 1 files, the first track is a tempo map */
    if (format == 1)
    {
	midi_writetempotrack();
    }

    /* The rest of the file is a series of tracks */
    for(i = 0; i < ntracks; i++)
        mf_write_track_chunk(i,fp);
}

void midi_file::mf_write_track_chunk(int which_track, FILE* fp)
{
	unsigned long trkhdr,trklength;
	long offset, place_marker;
	//void write16bit(),write32bit();
	
	
	trkhdr = MTrk;
	trklength = 0;

	/* Remember where the length was written, because we don't
	   know how long it will be until we've finished writing */
	offset = ftell(fp); 

#ifdef DEBUG
        printf("offset = %d\n",(int) offset);
#endif

	/* Write the track chunk header */
	write32bit(trkhdr);
	write32bit(trklength);

	Mf_numbyteswritten = 0L; /* the header's length doesn't count */

	    _handler->midi_writetrack(which_track);

	/* mf_write End of track meta event */
	eputc(0);
	eputc(meta_event);
	eputc(end_of_track);

 	eputc(0);
	 
	/* It's impossible to know how long the track chunk will be beforehand,
           so the position of the track length data is kept so that it can
           be written after the chunk has been generated */
	place_marker = ftell(fp);
	
	/* This method turned out not to be portable because the
           parameter returned from ftell is not guaranteed to be
           in bytes on every machine */
 	/* track.length = place_marker - offset - (long) sizeof(track); */

#ifdef DEBUG
printf("length = %d\n",(int) trklength);
#endif

 	if(fseek(fp,offset,0) < 0)
	    mferror("error seeking during final stage of write");

	trklength = Mf_numbyteswritten;

	/* Re-mf_write the track chunk header with right length */
	write32bit(trkhdr);
	write32bit(trklength);

	fseek(fp,place_marker,0);
} /* End gen_track_chunk() */


void midi_file::mf_write_header_chunk(int format, int ntracks, int division)
{
    unsigned long ident,length;
    //void write16bit(),write32bit();
    
    ident = MThd;           /* Head chunk identifier                    */
    length = 6;             /* Chunk length                             */

    /* individual bytes of the header must be written separately
       to preserve byte order across cpu types :-( */
    write32bit(ident);
    write32bit(length);
    write16bit(format);
    write16bit(ntracks);
    write16bit(division);
} /* end gen_header_chunk() */


/*
 * mf_write_midi_event()
 * 
 * Library routine to mf_write a single MIDI track event in the standard MIDI
 * file format. The format is:
 *
 *                    <delta-time><event>
 *
 * In this case, event can be any multi-byte midi message, such as
 * "note on", "note off", etc.      
 *
 * delta_time - the time in ticks since the last event.
 * type - the type of meta event.
 * chan - The midi channel.
 * data - A pointer to a block of chars containing the META EVENT,
 *        data.
 * size - The length of the meta-event data.
 */
int midi_file::mf_write_midi_event(
unsigned long delta_time,
unsigned int chan,
unsigned int type,
unsigned char *data,
unsigned long size)
{
    int i;
    //void WriteVarLen();
    unsigned char c;

    WriteVarLen(delta_time);

    /* all MIDI events start with the type in the first four bits,
       and the channel in the lower four bits */
    c = type | chan;

    if(chan > 15)
        perror("error: MIDI channel greater than 16\n");

    eputc(c);

    /* write out the data bytes */
    for(i = 0; i < (int) size; i++)
	eputc(data[i]);

    return(size);
} /* end mf_write MIDI event */

/*
 * mf_write_meta_event()
 *
 * Library routine to mf_write a single meta event in the standard MIDI
 * file format. The format of a meta event is:
 *
 *          <delta-time><FF><type><length><bytes>
 *
 * delta_time - the time in ticks since the last event.
 * type - the type of meta event.
 * data - A pointer to a block of chars containing the META EVENT,
 *        data.
 * size - The length of the meta-event data.
 */
int midi_file::mf_write_meta_event(
unsigned long delta_time,
unsigned char type,
unsigned char *data,
unsigned long size)
{
    int i;

    WriteVarLen(delta_time);
    
    /* This marks the fact we're writing a meta-event */
    eputc(meta_event);

    /* The type of meta event */
    eputc(type);

    /* The length of the data bytes to follow */
    WriteVarLen(size); 

    for(i = 0; i < (int) size; i++)
    {
	if(eputc(data[i]) != data[i])
	    return(-1); 
    }
    return(size);
} /* end mf_write_meta_event */

void midi_file::mf_write_tempo( unsigned long tempo )
{
    /* Write tempo */
    /* all tempos are written as 120 beats/minute, */
    /* expressed in microseconds/quarter note     */
    eputc(0);
    eputc(meta_event);
    eputc(set_tempo);

    eputc(3);
    eputc((unsigned)(0xff & (tempo >> 16)));
    eputc((unsigned)(0xff & (tempo >> 8)));
    eputc((unsigned)(0xff & tempo));
}

unsigned long midi_file::mf_sec2ticks(float  secs, int division, unsigned int tempo)
{    
     return (long)(((secs * 1000.0) / 4.0 * division) / tempo);
}

/*
 * Write multi-length bytes to MIDI format files
 */
void midi_file::WriteVarLen( unsigned long value)
{
  unsigned long buffer;

  buffer = value & 0x7f;
  while((value >>= 7) > 0)
  {
	buffer <<= 8;
	buffer |= 0x80;
	buffer += (value & 0x7f);
  }
  while(1){
       eputc((unsigned)(buffer & 0xff));
       
	if(buffer & 0x80)
		buffer >>= 8;
	else
		return;
	}
}/* end of WriteVarLen */

/* 
 * This routine converts delta times in ticks into seconds. The
 * else statement is needed because the formula is different for tracks
 * based on notes and tracks based on SMPTE times.
 *
 */
float midi_file::mf_ticks2sec( int division, unsigned int tempo, unsigned long ticks)
{
    float smpte_format, smpte_resolution;

    if(division > 0)
        return ((float) (((float)(ticks) * (float)(tempo)) / ((float)(division) * 1000000.0)));
    else
    {
       smpte_format = upperbyte(division);
       smpte_resolution = lowerbyte(division);
       return (float) ((float) ticks / (smpte_format * smpte_resolution * 1000000.0));
    }
} /* end of ticks2sec() */


/*
 * write32bit()
 * write16bit()
 *
 * These routines are used to make sure that the byte order of
 * the various data types remains constant between machines. This
 * helps make sure that the code will be portable from one system
 * to the next.  It is slightly dangerous that it assumes that longs
 * have at least 32 bits and ints have at least 16 bits, but this
 * has been true at least on PCs, UNIX machines, and Macintosh's.
 *
 */
void midi_file::write32bit( unsigned long data )
{
    eputc((unsigned)((data >> 24) & 0xff));
    eputc((unsigned)((data >> 16) & 0xff));
    eputc((unsigned)((data >> 8 ) & 0xff));
    eputc((unsigned)(data & 0xff));
}

void midi_file::write16bit(int data)
{
    eputc((unsigned)((data & 0xff00) >> 8));
    eputc((unsigned)(data & 0xff));
}

/* write a single character and abort on error */
int midi_file::eputc( unsigned char c )
{
	int return_val;
	
	return_val = midi_putc(c);

	if ( return_val == EOF )
		mferror("error writing");
		
	Mf_numbyteswritten++;
	return(return_val);
}


//@ write new style
void midi_file::write_header(int format, int ntracks, int division) 
{
    mf_write_header_chunk(format,ntracks,division);
}

int midi_file::begin_track(int which_track)
{
	if (_in_track) end_track();
	_in_track = 1;
	int the_track = which_track;
	if (!the_track) the_track = _trackno++;
	FILE* fp = out_file();
	
	trkhdr = MTrk;
	trklength = 0;

	/* Remember where the length was written, because we don't
	   know how long it will be until we've finished writing */
	offset = ftell(fp); 

#ifdef DEBUG
        printf("offset = %d\n",(int) offset);
#endif

	/* Write the track chunk header */
	write32bit(trkhdr);
	write32bit(trklength);

	Mf_numbyteswritten = 0L; /* the header's length doesn't count */

	    _handler->midi_writetrack(the_track);

	return 1;
	}

/// mf_write End of track meta event 
int midi_file::end_track() {
	if (!_in_track) return 0;
	_in_track = 0;
	FILE* fp = out_file();
	eputc(0);
	eputc(meta_event);
	eputc(end_of_track);

 	eputc(0);
	 
	/* It's impossible to know how long the track chunk will be beforehand,
           so the position of the track length data is kept so that it can
           be written after the chunk has been generated */
	place_marker = ftell(fp);
	
	/* This method turned out not to be portable because the
           parameter returned from ftell is not guaranteed to be
           in bytes on every machine */
 	/* track.length = place_marker - offset - (long) sizeof(track); */

#ifdef DEBUG
printf("length = %d\n",(int) trklength);
#endif

 	if(fseek(fp,offset,0) < 0)
	    mferror("error seeking during final stage of write");

	trklength = Mf_numbyteswritten;

	/* Re-mf_write the track chunk header with right length */
	write32bit(trkhdr);
	write32bit(trklength);

	fseek(fp,place_marker,0);
	return 1;
} // end_track


void midi_file::write_event(int delta, int chan, int type, const  char* data, int size) {
	//cerr << "midi_file::write_event " << (int) data[0] << endl;
	mf_write_midi_event((unsigned long)delta, (unsigned int) chan, (unsigned int) type, (unsigned char*) data, (unsigned long) size);
	}
void midi_file::write_meta(int delta, int type, const  char* data, int size) {
	mf_write_meta_event((unsigned long)delta, (unsigned int) type, (unsigned char*) data, (unsigned long) size);
	}


void midi_file::write_tempo(int x) {
	mf_write_tempo((long)(x * 4166));
	}
