#include <hush/source.h>
#include <hush/ml/format.h>
#include <hush/ml/term.h>
#include <hush/string.h>

#include <hush/ml/format.i>

#include <hush/iostream.h>
#include <stdio.h>
#include <string.h>

static char formatbuf[1024*12];

static string* dummy = 0;

#define TYPE formatter
#include "hush/sys/table.i"

dictionary<dvstring>* formatter::_defaults = 0;
formatter* formatter::_default = 0;
handler*   formatter::_garbage = 0;

void (*formatter::_init)(dictionary<dvstring>&) = 0;

void formatter::_hold(const handler* x) {
        handler *h = formatter::_garbage;
        if (!h) {
                h = formatter::_garbage = new handler();
                if (session::_default) session::_default->_register(h);
                }
        if (x) h->_register(x);
        }

void formatter::_cleanup() {
        handler *h = formatter::_garbage;
        if (h) h->_cleanup();
	//delete h;
	formatter::_garbage = 0;
        }

char* formatter::_buf() { return formatbuf; }

int formatter::initialized = 0;

int formatter::initialize() {
	if (!formatter::initialized) {
		formatter::initialized = 1;
		formatter::_defaults = new dictionary<string>(1); // auto
	        formatter* x = new term_formatter(); 
		formatter::declare("term",x);
		formatter::declare("ml",x);
		if (!formatter::_default) 
		    formatter::_default = x;
		if (session::_default) {
	 	    session::_default->_register(formatter::_defaults);
	 	    session::_default->_register(x);
                } else {
  		   formatter::_hold(0);
		   formatter::_garbage->_register(formatter::_defaults);
		   formatter::_garbage->_register(x);
		} 
		if (!formatter::_init)
			formatter::_init = default_formats;
		dictionary<string>& d = *formatter::_defaults;
                formatter::_init(d);
		}
	return formatter::initialized = 1;
	}

void formatter::defaults(const char* k, const char* fm) {
	(*formatter::_defaults)[k] = new string(fm);
	}

formatter::formatter(char* k) {
        if (k); //Wall
	_formats = 0;
	if (!formatter::initialize()) {
		cerr << "ERROR: initializing formatter" << endl;
		}
	if (!::dummy) {
		dummy = new string("");
		_register(dummy);
		}
	formatter* x = formatter::lookup(k);
	if (!x) {
		//x = tk->formatter(k);  load---
		x = formatter::_default;
		}
	_formatter = x;
	}

char* formatter::get(const char* k, int arity) {
	char* fm = 0; if (!k) return fm;
	//AE: if (_formatter) fm = _formatter->get(k,arity);
	if (fm) return fm;
	char buf[256]; string* p;
	if (arity >= 0) sprintf(buf,"%s/%d",k,arity);
	else sprintf(buf,"%s",k);
	if (!fm && _formats) {
	        p = (*_formats)[buf];
                if (p) fm = (char*) *p;
		}
	if (fm) return fm;
	dictionary<string>& d = *(formatter::_defaults);
        p = d[buf];
	if (!p) {
		sprintf(buf,"%s",k);
		p = d[buf];
		}
        if (p) fm = (char*) *p;
	return fm;
	}

void formatter::format(const char* k, const char* fm) {
	if (0 && _formatter) _formatter->format(k,fm);
	else if (!_formats) {
			_formats = new dictionary<string>(1); // auto 
	                (*_formats)[k] = new string(fm);
			_register(_formats);
		}
	else if (fm && fm[0]!='\0') (*_formats)[k] = new string(fm);
	}

char* formatter::_format(const term* t) { 
	cerr << "Invoking _format" << endl;
	if (_formatter && _formatter != this)
		return _formatter->_format(t);
	else return 0;
	}

term_formatter::term_formatter() { 
	//cerr << "Create term formatter" << endl;
	}

char* term_formatter::_format(const term* t) {
   //cerr << "[*]";
   string* r = 0;
   char* fm = 0;
   if (!t) {
	cerr << "Error: (NULL) term" << endl; 
	r = new string("error");
   } else if (!t->v && t->s) {
	fm = get(t->s,0);
	if (!fm) fm = t->s;
	sprintf(formatbuf,fm);
	r = new string(formatbuf);
   } else if (t->_ismeta() && (t->nargs()==0)) {
	fm = get(t->fun(),0);
	if (!fm) fm = get("#default:",0);
	sprintf(formatbuf,fm);
	r = new string(formatbuf);
   } else if (t->_ismeta() && (t->nargs()==1)) {
	char* s1 = t->a1()->str(this);
	fm = get(t->fun(),1);
	if (!fm) fm = get("#default:",1);
	sprintf(formatbuf,fm,(char*)s1);
	r = new string(formatbuf);
   } else if (t->_ismeta() && (t->nargs()==2)) {
	char* s1 = t->a1()->str(this);
	char* s2 = t->a2()->str(this);
	fm = get(t->fun(),2);
	if (!fm) fm = get("#default:",2);
	sprintf(formatbuf,fm,(char*)s1,(char*)s2);
	r = new string(formatbuf);
   } else if (!t->_isfun()) {
	fm = get(t->fun(),0);
	if (fm) r = new string(fm);  // an alias, basically
	else r = new string(t->fun());
   } else if (t->_opin(":+-/*=") && t->a1() && t->a2()) {
	char* s1 = t->a1()->str(this);
	char* s2 = t->a2()->str(this);
	fm = get(t->fun(),2);
	if (fm) sprintf(formatbuf,fm,(char*)s1,(char*)s2);
	else	
	   sprintf(formatbuf,"%s %s %s",(char*)s1,t->fun(),(char*)s2);
	r = new string(formatbuf);
   } else if (t->_isfun(",") && t->a1() && t->a2()) {
	char* s1 = t->a1()->str(this);
	char* s2 = t->a2()->str(this);
	fm = get(t->fun(),2);
	if (fm) sprintf(formatbuf,fm,(char*)s1,(char*)s2);
	else	
	  sprintf(formatbuf,"%s, %s",(char*)s1,(char*)s2);
	r = new string(formatbuf);
   } else if (t->_isfun(",") && t->a1()) {
	char* s1 = t->a1()->str(this);
	sprintf(formatbuf,"%s",(char*)s1);
	cerr << "WARNING: single comma: " << (char*)*s1 << endl;
	r = new string(formatbuf);
   } else if ((t->_isfun("!") || t->_isfun("<-")) && (t->nargs()==2)) {
	char* s1 = t->a1()->str(this);
	char* s2 = t->a2()->str(this);
	fm = get(t->fun(),2);
	if (fm) sprintf(formatbuf,fm,(char*)s1,(char*)s2);
	else	
	  sprintf(formatbuf,"%s <- %s",(char*)s1,(char*)s2);
	r = new string(formatbuf);
   } else if (t->a1() && t->a2()) {
   	char* s1 = t->a1()->str(this);
   	char* s2 = t->a2()->str(this); 
	fm = get(t->fun(),2);
	if (fm) sprintf(formatbuf,fm,(char*)s1,(char*)s2);
	else	
   	  sprintf(formatbuf,"%s(%s, %s)", t->fun(),(char*)s1,(char*)s2);
	r = new string(formatbuf);
   } else if ( t->a1() ) {
   	char* s1 = t->a1()->str(this);
	fm = get(t->fun(),1);
	//cerr << "**"  << fm << endl;
	if (fm) sprintf(formatbuf,fm,(char*)s1);
	else	
	  sprintf(formatbuf,"%s(%s)",t->fun(),(char*)s1 );
	r = new string(formatbuf);
   } else {
                //cerr << "ERROR: formatter" << endl;
		char* p = t->fun();
		if (p)
			r = new string(p);
		else r = new string("error");
   }
   //if (fm) cerr << "<<" << (char*) fm << ":" << (char*)*r << ">>";
   t->_register(r);
   return (char*) *r;
   }
