// http_client.c
//
// implementation file for the HTTP netclient, using HTTP/1.0
// http_client is a child of netclient
// based on Jacco's http_client
//
// October 1995
// Bastiaan Sch\"onhage
//
// January 1997
// Adapted for Windows 95
// Jacco van Ossenbruggen

#include <hush/source.h>

#include <hush/iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <hush/sys/unistd.h>

#ifndef __MSWIN__
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#else 
#include <winsock.h>
static int wsa_errno;
static WSADATA wsadata; // for WSAStartup()
#define perror(str) cerr << str << " WSA/errno: "\
	<< strerror(WSAGetLastError()) << "/" \
	<< strerror(errno) << endl; 
#define read(fd,buf,len)  recv(fd,buf,len,0)		// dangereous!!!
#define write(fd,buf,len) send(fd,buf,len,0)
#endif 

#include <sys/types.h>

#include <hush/net/client/http_client.h>
#include <hush/net/client/url.h>

const char SP	= ' ';
const char CR	= '\r';
const char LF	= '\n';
const char EOS	= '\0';
const int bufsize = 8192;
const int max_line_length = 256;

http_client::http_client(int tracelevel):
client(0),
reply_(NULL),
reply_header_(NULL),
got_reply_(false),
parsed_reply_(false),
sd_is_body_(false),
url_(NULL),
sd_(-1),
reply_length_(0),
reply_header_length_(0),
max_reply_length_(bufsize),   // will be automaticly enlarged when needed 
errorcode_(0),
tracelevel_(tracelevel) 
{
	reply_ = new char[max_reply_length_];
	request_header_ = new char[bufsize];
	request_header_[0] = 0;

#ifdef __MSWIN__
	// init winsock api
	if (WSAStartup(MAKEWORD(1,1), &wsadata))
		cerr << "Error: could not initialize winsock API" << endl;
#endif
	if (tracelevel_ > 10) cerr << "Trace: http_client created"  << endl;
}

http_client::~http_client() {
	clean_up();
}

const char* http_client::type(void) const
{
	return "http";
}

void http_client::clean_up() {
	if (reply_) delete[] reply_;
	if (reply_header_) delete[] reply_header_;
	if (request_header_) delete[] request_header_;
	if (url_) delete url_;
	if (sd_ > 0) {
		if (tracelevel_) cerr << "Trace: Client closed " << sd_ << endl;
		close(sd_);
		sd_ = -1;
	}
}

void http_client::trace(int t) { tracelevel_ = t; }
int http_client::tracelevel() const { return tracelevel_; }

void http_client::get(const char* URL)
{
	if (url_) delete url_;
	url_ = new url(URL);
}

void http_client::header(const char* attribute, const char* value) {
	strcat(request_header_, attribute);
	strcat(request_header_, ": ");
	strcat(request_header_, value);
	strcat(request_header_, "\n");
	if (tracelevel_) cerr << attribute << ": " << value << endl;
}

void http_client::clear_header() {
	request_header_[0] = 0;
}

bool http_client::send_request() {
	if (sd_ > 0) close(sd_); sd_ =-1;
	if (!url_) return false;
	char *request_get = new char[strlen(url_->locator())+ 64];
	sprintf(request_get, "GET /%s HTTP/1.0\n", url_->locator());
	if (tracelevel_) cerr << request_get << flush;
	int length = strlen(request_get) +	strlen(request_header_) + 2;
	char *request = new char[length];
	strcpy(request, request_get);
	strcat(request, request_header_);
	strcat(request, "\n");
	delete[] request_get;
	bool r = send_request(url_->host(), url_->port(), request, strlen(request));
	delete[] request;
	return r;
}

bool http_client::send_request(const char* hostname, int port, const char *request, int length) {
	got_reply_ = false;
	// check hostname;
	hostent *hp = gethostbyname(hostname);
	if (!hp) {
		cerr << "Unknown host: " << hostname << endl;
		return false;
	}
	sockaddr_in name; // Internet sock addr
	memcpy(&name.sin_addr.s_addr, hp->h_addr, hp->h_length);
	name.sin_port = htons(port);
	name.sin_family = AF_INET;
	
	if ((sd_ = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
		perror("Error: client could not create TCP/IP socket");
		sd_ = -1;
		return false;
	} else if (tracelevel_ > 5) 
		cerr << "Trace: client created socket on " << sd_ << endl;

	if (connect(sd_, (sockaddr*)&name, sizeof(name)) < 0) {
		perror("Error: client could not connect");
		close(sd_); sd_ = -1;
		return false;
	} else if (tracelevel_ > 5) 
		cerr << "Trace: client has connection with peer " <<
			hostname << ":" << port << endl;
	if (length != write(sd_, request, length)) {;
		cerr << "Error: Could not send all bytes requested" << endl;
		close(sd_); sd_ = -1;
		return false;
	} else if (tracelevel_ > 5)  
		cerr << "Trace: send request" << endl << request << endl;
	got_reply_ = false;
	sd_is_body_ = false;
	return true;
}
bool http_client::get_reply(char* &reply_out, int &length) {
	if (sd_ < 0) return false;
	int nbytes=0, start=0, size=max_reply_length_;
	char *new_reply = reply_;
	while((nbytes = read(sd_, &new_reply[start], size)) > 0) {
		start += nbytes;
		size = max_reply_length_ - start;
		if (size == 0) {
			// Oops. The buffer is full. Double its size
			start = size = max_reply_length_;
			max_reply_length_ *= 2;
			new_reply = new char[max_reply_length_];
			memcpy(new_reply, reply_, size);
			delete[] reply_;
			reply_ = new_reply;	    
			if (tracelevel_ > 5) {
				cerr << "Trace: the reply buffer will be enlarged to " << 
					max_reply_length_ << endl;
			}
		}
	}
	reply_[start]=EOS;		// could be buggy
	reply_out = reply_;
	reply_length_ = start;
	length = start;
	got_reply_ = true;
	parsed_reply_ = false;
	return true;
}


inline void http_client::skip(char* &ptr, char c)	// skip till + including char
{
	while(*ptr != c) ptr++;
	ptr++;
}


bool http_client::parse_reply(void)
{
	char *start, *ptr;
	char type_[100];
	char subtype_[100];

	int i=0;

	if(!got_reply_) 
		if(!get_reply(reply_, reply_length_)) return false;
	if(parsed_reply_) return true;
	start=ptr=reply_;
	content_length_ = 0;

	// parse header
	if (strncasecmp(reply_,"HTTP",4)==0) 		// parse header
	{
		skip(ptr, SP);
		errorcode_ = atoi(ptr);
		skip(ptr, LF);
		while((*ptr != CR) && (*ptr != LF)) 
		{
			if(strncasecmp(ptr, "Content-type:", 13)==0) {
				ptr+=13;
				while(*ptr == SP) ptr++;
				i=0;
				while(*ptr != '/') type_[i++]=*ptr++;
				type_[i]=EOS;
				ptr++;
				i=0;
				while((*ptr != CR) && (*ptr != LF)) 
					subtype_[i++]=*ptr++;
				subtype_[i]=EOS;
				skip(ptr, LF);
				mime_.type(type_);
				mime_.subtype(subtype_);
			}
			else if(strncasecmp(ptr, "Content-length:", 15)==0) {
				ptr+=15;
				content_length_ = atoi(ptr);
				skip(ptr, LF);
			}
			else 
				skip(ptr, LF);
		}
		skip(ptr, LF);
	}

	
	// copy the header
	if(reply_header_) delete[] reply_header_;
	reply_header_length_ = ptr - start;
	reply_header_ = new char[reply_header_length_+1];
	memcpy(reply_header_, reply_, reply_header_length_);
	reply_header_[reply_header_length_] = EOS;

	parsed_reply_ = true;
	return true;
}


bool http_client::get_reply_header(char* &reply_out, int &length)
{
	if (parse_reply())
	{
		reply_out = reply_header_;
		length = reply_header_length_;
		return true;
	}
	return false;
}

bool http_client::get_reply_body(char* &reply_out, int &length)
{
	if (parse_reply())
	{
		reply_out = reply_+reply_header_length_;
		length = reply_length_ - reply_header_length_;
		return true;
	}	
	return false;
}

bool http_client::get_reply_trailer(char* &, int &)
{
	return false;
}

char * http_client::getline(char* line)
{
	char c;
	int i = 0;

	do
	{
		read(sd_, &c, 1);
		if (i<max_line_length) line[i++] = c;
		else cerr << "http_client::getline line too long"<< endl;
	} while ( c != LF );
	return line;	
}


bool http_client::get_reply()		// get header
{
	char *ptr;
	char type_[100];
	char subtype_[100];
	
	int i;

	if ( sd_<0 ) return false;

	char* line = new char[max_line_length];	
	if ( (ptr = getline(line)) == NULL) return false;
	
	// parse header
	if (strncasecmp(ptr, "HTTP",4)==0) 		// parse header
	{
		skip(ptr, SP);
		errorcode_ = atoi(ptr);
		if ( (ptr = getline(line)) == NULL) 
			return false;
		while((*ptr != CR) && (*ptr != LF)) 
		{
			if(strncasecmp(ptr, "Content-type:", 13)==0) {
				ptr+=13;
				while(*ptr == SP) ptr++;
				i=0;
				while(*ptr != '/') type_[i++]=*ptr++;
				type_[i]=EOS;
				ptr++;
				i=0;
				while((*ptr != CR) && (*ptr != LF)) 
					subtype_[i++]=*ptr++;
				subtype_[i]=EOS;
				mime_.type(type_);
				mime_.subtype(subtype_);
			}
			else if(strncasecmp(ptr, "Content-length:", 15)==0) {
				ptr+=15;
				content_length_ = atoi(ptr);
			}
			if ( (ptr = getline(line))==NULL) 
				return false;
			
		}
	}

	delete[] line;
	sd_is_body_ = true;
	parsed_reply_ = true;

	return true;
}

bool http_client::get_reply_body( int &fd )
{
	if (sd_is_body_ && (sd_ >=0) ) 
	{
		fd = sd_;
		return true;
	}
	else return false;
}


MIME* http_client::media()
{
	if ( parsed_reply_ )
	{
		return &mime_;
	}
	else return 0;
}

int http_client::length(void)
{	
	if ( parsed_reply_ ) return content_length_;
	else return 0;
}

int http_client::errorcode(void)
{
	if ( parsed_reply_ ) return errorcode_;
	else return 0;
}

