// url.c
//
// simple implementation for a url class
// based on Jacco's url class
//
// October 1995
// Bastiaan Sch\"onhage

#include <hush/source.h>

#include<hush/iostream.h>
#include<stdlib.h>
#include<stdio.h>
#include<string.h>


#include <hush/net/client/url.h>
//naive implementation of url.h

static const int default_port			= 80;  //AE 80;
static const char* const default_protocol 	= "any"; // ""; //AE "http";
static const int max_protocol_len 		= 12; //6; // strlen("gopher");

url::url():
	protocol_(NULL), host_(NULL), locator_(NULL),
	port_(default_port), s_(NULL), es_(NULL)  
{
	init("http://www.cs.vu.nl/~hush/");
}

url::url(const char* const s): 
	protocol_(NULL), host_(NULL), locator_(NULL),
	port_(default_port),s_(NULL), es_(NULL)
{
	init(s);
}

url::url(const url& u): 
	protocol_(NULL), host_(NULL), locator_(NULL),
	port_(u.port_), s_(NULL), es_(NULL)  
{
	if (u.protocol_) 
		strcpy(protocol_ = new char[strlen(u.protocol_)+1], u.protocol_);
	if (u.host_)
		strcpy(host_     = new char[strlen(u.host_)    +1], u.host_);
	if (u.locator_)
		strcpy(locator_	 = new char[strlen(u.locator_) +1], u.locator_);
	if (u.s_)
		strcpy(s_        = new char[strlen(u.s_)       +1], u.s_);
}


url::~url() {
	clean_up();
}

url::operator char* () const {
	if (!s_) ((url*)this)->makestring();
	return s_;
}


const char* url::expand(const char *string)
{
	if (!s_) makestring();
	if (es_) delete[] es_;
	es_ = new char[strlen(s_) + strlen(string) +1];

	if (strchr(string, ':') != 0)		// string is full url
		strcpy(es_, string);
	else
	{
		strcpy(es_, s_);
		char *ptr = strrchr(es_, '/');	// search last '/'
		if(ptr) *(ptr+1)='\0';		// add after the last '/'
		strcat(es_, string);
	}
	return es_;
}


url& url::operator=(const url& u) {
	if (this != &u) {
		clean_up();
		port_ = u.port_;
		if (u.protocol_) 
			strcpy(protocol_ = new char[strlen(u.protocol_)+1], u.protocol_);
		if (u.host_)
			strcpy(host_ = new char[strlen(u.host_) +1], u.host_);
		if (u.locator_)
			strcpy(locator_ = new char[strlen(u.locator_)+1], u.locator_);
		if (u.s_)
			strcpy(s_ = new char[strlen(u.s_)+1], u.s_);
//		if (u.es_)
//			strcpy(es_ = new char[strlen(u.es_)+1], u.es_);
	}
	return *this;
}



void url::clean_up() {
	if (protocol_) delete[] protocol_;
	if (host_)     delete[] host_;
	if (locator_)  delete[] locator_;
	if (s_)        delete[] s_;
	if (es_)       delete[] es_;
	protocol_ = host_ = locator_ = s_ = es_ = 0;
}

const char* url::protocol() const {
	return protocol_;
}

const char* url::host() const {
	return host_;
}

const char* url::locator() const {
	return locator_;
}

int url::port() const {
	return port_;
}

void url::init(const char* const s) {
	clean_up();
	int len = strlen(s); 
	char *buf = new char[len+1];
	strcpy(buf, s);
	char *p=buf; int i=0;
	while(*p++ != ':' && i< max_protocol_len && i<len) i++;
	if (i == max_protocol_len || i==len) {
		p=buf; i=0;
		strcpy(protocol_ = new char[strlen(default_protocol) +1], 
		       default_protocol);
	} else {
		buf[i]=0;
		protocol_ = new char[i+1];
		strcpy(protocol_, buf);
	}
	
	while(strchr("/:", *p) != 0) p++; 
	char *start_host = p;
	
	int use_default_port = 1;
	i=0; len = strlen(p);
	while(i<len && strchr("/:", *p++) == 0) i++;
	if (i==len) {
		host_ = new char[i+1];
		strcpy(host_, start_host);
		locator_ = new char [i+1];
		strcpy(locator_, "");
	} else {
		if (start_host[i] == ':') use_default_port = 0;
		start_host[i]=0;
		
		host_ = new char[i+1];
		strcpy(host_, start_host);
		
		if (!use_default_port) {
			char *start_port = p;
			i=0;
			while(*p && *p++ != '/') i++;
			start_port[i]=0;
			port_ = atoi(start_port);
		} else port_ = default_port;
		locator_ = new char[strlen(p)+1];
		strcpy(locator_, p);
	}
	delete[] buf;
	makestring();
}

void url::makestring() {
	if(s_) delete[] s_; 
	char port_string[12];
	if (port_ != default_port) sprintf(port_string, ":%d/", port_);
	else sprintf(port_string, "/");
	
	s_ = new char[strlen(protocol_) +
		      strlen("://") +
		      strlen(host_) +
		      strlen(port_string) +
		      strlen(locator_)+1];
	strcpy(s_, protocol_);
	strcat(s_, "://");
	strcat(s_, host_);
	strcat(s_, port_string);
	strcat(s_, locator_);

}

// Emacs:
// Local Variables:
// mode: C++
// End: 
