#include <hush/source.h>

#include <hush/dictionary.h>

#include <hush/iter.h>
#include <hush/string.h>
#include <hush/garbage.h>

#include <hush/sys/cell.h>

#undef string

extern "C" {            // the hashtable is taken from Tcl
#include "hush/sys/hash.i"
}

//#include <iostream.h>

#include "hush/sys/local.i"
#include "hush/sys/macros.i"

#undef string

//int dictionary_type::created = 0;
//int dictionary_type::deleted = 0;

dictionary_type::dictionary_type() { created++; }
dictionary_type::~dictionary_type() { deleted++; }


#define TYPE dictionary_type
#define DESTROY _destroy
#include "hush/sys/table.i"

void dictionary_api::set_dictionaryentry(void* t, char* k, void* v) {
   rawdictionary* raw = (rawdictionary*) t;
   if (raw->_auto) {
	raw->_auto->_put(v);
	raw->_auto->operator()();
	//AE delete raw->_auto;
	raw->_auto = 0;
	}
   int n = -1;
   Hush_HashEntry* p;
        do {
                p = Hush_CreateHashEntry((Hush_HashTable*)raw->tbl,k, &n);
   } while( n < 0 );

   
   if (p) Hush_SetHashValue( (Hush_HashEntry*) p, (ClientData) v);
   }

void* dictionary_api::get_dictionaryentry(void* t, char* k) {
   rawdictionary* raw = (rawdictionary*) t;
   if (raw && raw->_auto) {
	delete raw->_auto; raw->_auto = 0;
	}
   if (!k || !raw) return 0;
   Hush_HashEntry* p = !k?0:Hush_FindHashEntry((Hush_HashTable*)raw->tbl,k);
   if (!p) return 0;
   else return Hush_GetHashValue( (Hush_HashEntry*) p);
   }


rawdictionary::rawdictionary() {
	_auto = 0;
	tbl = new Hush_HashTable;
	Hush_InitHashTable((Hush_HashTable*)tbl,TCL_STRING_KEYS);
	_garb = 0; _sgarb = 0; _egarb = 0; _igarb = 0; _lgarb = 0;
	}

rawdictionary::~rawdictionary() {
	Hush_DeleteHashTable((Hush_HashTable*)tbl);
	HUSHDELETE(_garb,dvgarbage<void>);
	HUSHDELETE(_igarb,iter<void>);
	HUSHDELETE(_sgarb,dvstring);
	HUSHDELETE(_egarb,dictionaryentry<void>);
	//DELETE(_lgarb,list<void>);
	}

void rawdictionary::i_register(void* p) { REGISTER( _igarb, p); }
void rawdictionary::s_register(void* p) { REGISTER( _sgarb, p); }
void rawdictionary::e_register(void* p) { REGISTER( _egarb, p); }
void rawdictionary::_register(void* p) { REGISTER(_garb, p); }


void rawdictionary::entry(char* k, void* v) {
	int n;
	Hush_HashEntry* p;
	do {
		p = Hush_CreateHashEntry((Hush_HashTable*)tbl,k, &n);
	} while( !n );
	Hush_SetHashValue(p, (ClientData) v);
	}

void* rawdictionary::find(char* k) {
	if (!k) k = "illegal-key";
	Hush_HashEntry* p;
	p = Hush_FindHashEntry((Hush_HashTable*)tbl,k);
	if (!p)   { 
		  return 0; 
		  }
	else return Hush_GetHashValue(p);
	}

void* rawdictionary::del(char* k) {
	Hush_HashEntry* p;
	if (k) p = Hush_FindHashEntry((Hush_HashTable*)tbl,k);
	if (!p) {
		return 0;
		}
	void* v =  Hush_GetHashValue(p);
	Hush_DeleteHashEntry( p );
	return v;
	}

char* rawdictionary::stats() {
	return Hush_HashStats( (Hush_HashTable*)tbl );
	}
	
void* rawdictionary::operator()(char* k) { return find(k); }

dictionaryentry<void>*
rawdictionary::operator[](char* k) {  /// trick to allow assignment
	/*
	int n = -1;
	Hush_HashEntry* p;
	do {
		p = Hush_CreateHashEntry((Hush_HashTable*)tbl,k, &n);
	} while( n < 0 );
	*/
	return  new dictionaryentry<void>( this, k );
	}
	



class rawdictionaryiter : public iter<dvstring> {
int _peek;
public:
rawdictionaryiter(Hush_HashTable* tbl_, rawdictionary* d_) 
						: iter<dvstring>(this) {
	_peek = 0;
	tbl = tbl_; sp = &s; d = d_;
	p = Hush_FirstHashEntry(tbl,sp);
	//if (p) cout << "oke"; else cout << "wrong"; cout << endl;
	}


						~
rawdictionaryiter() { }

void peek() { _peek = 1; }
void dummy(void*) { }

dvstring* operator()() {
	//if (p) cout << "oke"; else cout << "wrong"; cout << endl;
	dvstring* k = 0;
	if (!p) return 0;
	char* v = p ?  Hush_GetHashKey(tbl,p) : 0;
	if (!_peek) k = v? new dvstring(v): 0;
	else k = (dvstring*) v;
	//d->s_register(k);
	if (p) p = Hush_NextHashEntry(sp);
	return k;
	}

	
private:
Hush_HashSearch* sp;
Hush_HashTable* tbl;
Hush_HashSearch s;
Hush_HashEntry* p;
rawdictionary* d;
};


iter<dvstring>* rawdictionary::search() {
iter<dvstring>* tmp =  new rawdictionaryiter((Hush_HashTable*)tbl,this); 
i_register((void*)tmp);
return tmp;
}


void rawdictionary::destroy() {
	iter<dvstring>& it = *search();
	dvstring* p = 0;
	while ( (p = it()) )
			del((char*)*p);
	}
