#include <hush/source.h>

#include <hush/string.h>
#include <hush/event.h>

#include <hush/sys/cell.h>

#include <hush/flags.h>
#ifndef NOREGEX
#include <hush/sys/regexp.h>
#endif

#include "hush/sys/local.i"
#include "hush/sys/macros.i"

#include <stdio.h>

//#include <iostream.h>

int string::created = 0;
int string::deleted = 0;

int substring::created = 0;
int substring::deleted = 0;

const int MaxWArgs = 36;
const int MaxLArgs = 256;


class stringrep {
friend class string;
friend class substring;
friend class string_iter;
private:
stringrep(const char* s);
stringrep(int argc_,  char* argv_[]);
stringrep(int argc_, char* argn_[],  char* argv_[]);

int length();
char* flatten(int i);
int unflatten();
void convert();
int isassoc();
void mkassoc();

void u_register(substring* s);
void s_register(string* s);
void l_register(cell* );
void i_register(iter<string>*);

void split(const char* s);

~stringrep();
char* rep;
int len;
int refs;
int _argc;
char** _argn;
char** _argv;
char* _key;
char* _value;
int _mode;
cell* _list;
cell* _garb;
cell* _lgarb;
cell* _igarb;
cell* _ugarb;
};

#define REP ((stringrep*)_body)
#define SREP ((stringrep*) ((string*)_body)->_body)

substring::substring(string* x,int p,int l) : _body(x), _str(0) {
	substring::created++;
	x->_register((substring*)this);
	pos = p; len = l;
	}

substring::~substring() {
	substring::deleted++;
	}

string& substring::operator=( char c ) {
	SREP->rep[pos] = c;
	return *this;
	}

// AE : leaks -- !!

string& substring::operator=( const char* s ) {
	_str = (char*) s; //AE
	len = len?len:SREP->length() - pos;
	int ns = strlen(s);
	int n = SREP->length() - len + ns;
	_str = new char[n+1];
	int i = 0;
	while ( i < pos ) {
		_str[i] = SREP->rep[i];
		i++;
		}
	int j = 0;
	while ( j < ns ) {
		_str[i] = s[j];
		i++; j++;
		}
	int k = pos + len;
	while ( k < SREP->length() ) {
		_str[i] = SREP->rep[k];
		k++; i++;
		}
        _str[i] = '\0';
	SREP->refs--;
	((string*) _body)->_body = new stringrep(_str);
	delete _str; _str = 0; // AE
	return *(string*) _body;
	}

//AE substring::operator char() { return SREP->rep[pos]; }

substring::operator char*() { // AE - leaks
	if (!len) len = SREP->length() - pos;
        _str = new char[len+1];
	strncpy(_str, (SREP->rep)+pos, len );
	_str[len] = '\0';
	return _str;
	}

substring::operator string&() { // AE -- leak
        _str = new char[len+1];
	strncpy(_str, (SREP->rep)+pos, len );
	_str[len] = '\0';
	string* s = new string(_str);
	delete _str; _str = 0; // AE -- leak
	return *s;
	}

class string_iter : public iter<string> {
public:
string_iter(stringrep* r);
string* operator()();
private: 
int n;
stringrep* _r;
};

//// These are friends

int operator==( const char* s, const dvstring& a ) { return a == s; }
int operator!=( const char* s, const dvstring& a ) { return a != s; }

string operator+( const char* s, const dvstring& a ) { 
        dvstring* x = new dvstring(s);
        a._register(x);
        return *x + a;
        }

string operator+( const char c, const dvstring& a ) { 
	char buf[2]; buf[0]=c; buf[1]='\0';
        dvstring* x = new dvstring(buf);
        a._register(x);
        return *x + a;
        }

//// And now for strings 

string::string() {
	string::created++;
	_body = (void*) new stringrep("");
	}

string::~string() {
	string::deleted++;
	if (REP->refs == 0) delete REP;
	else REP->refs--;
	}

string::string(const char* s) {
	string::created++;
	_body =  new stringrep(s);
	}

string::string(const string& a) { 
	   string::created++;
	   _body = (void*) new stringrep( (char*) a );
	   //_body = a._body; REP->refs++;   // sharing is obsolete
	   } 

string::string(int argc_, char** argv_) {
	string::created++;
	_body = new stringrep(argc_,argv_);
	}

string::string(int argc_, char** argn_, char** argv_) {
	string::created++;
	_body = new stringrep(argc_,argn_,argv_);
	}

string::string(const event* e) {
       string::created++;
       _body = new stringrep(e->argc(), e->argv());
       }

string& string::clone() {
	char* p = (char*) this;
	string* x = new string(p);
	return *x;
	}

string& string::operator=( const string& a ) { 
	   if (_body != a._body) {
	   	REP->refs--;
	   	if (REP->refs <= 0 )
			delete REP;
		
	   _body = a._body;
	   REP->refs++;
	   }
	   return *this;
	   }

string& string::operator=( const char* a ) {
	   if (0 && _body ) {
	   	REP->refs--;
	   	if (REP->refs <= 0 )
			delete REP;
	        }
	   _body = new stringrep(a);
	   return *this;
	   }

int string::length() const { return REP->length(); } 

string::operator char*() const { return REP->rep; }

substring& string::operator[](int p) {
	substring* tmp = new substring(this,p,0);
	return *tmp;
	}

substring& string::operator()(int p, int len) {
	substring* tmp = new substring(this,p,len);
	return *tmp;
	}

string& string::_substring(int i, int lng) {
	int n = lng;
	char* tmp = new char[n+1];
	for( int x = 0; x < n; x++ ) tmp[x] = (REP->rep)[i+x];
	tmp[n] = 0;
	string* result = new string( tmp );
	_register(result);
        delete tmp;
	return *result;
	}

string string::operator+( const string& a ) const { 
	char* tmp = new char[length() + a.length() + 1];
	strcpy(tmp,REP->rep); strcat(tmp, (char*) a);
	string* result = new string( tmp );
	((string*)this)->_register(result);
	delete tmp;
	return *result;
	}

string string::operator+( const char* a ) const { 
	char* tmp = new char[length() + strlen(a) + 1];
	strcpy(tmp,REP->rep); strcat(tmp, a);
	string* result = new string( tmp );
	((string*)this)->_register(result);
	delete tmp;
	return *result;
	}

string string::operator+( const char c ) const {
	char buf[2]; buf[0] = c; buf[1] = '\0';
	return *this + buf;
	}

string& string::operator+=( const string& a ) { 
	char* tmp = new char[length() + a.length() + 1];
	strcpy(tmp,REP->rep); strcat(tmp, (char*) a);
	(*this) = tmp;
	delete tmp;
	return *this;
	}

string& string::operator+=( const char* a ) { 
	char* tmp = new char[length() + strlen(a) + 1];
	strcpy(tmp,REP->rep); strcat(tmp, a);
	(*this) = tmp;
	delete tmp;
	return *this;
	}

string& string::operator+=( const char c ) {
	char buf[2]; buf[0] = c; buf[1] = '\0';
	return *this += buf;
	}

string& string::operator<<( const string& a ) {
	*this += " ";
	return *this += a;
	}

string& string::operator<<( const char* s ) {
	*this += " ";
	return *this += s;
	}

string& string::operator<<( const char c ) {
	*this += " ";
	return *this += c;
	}

string& string::operator<<( const int i ) {
	char buf[24];
	sprintf(buf," %d",i);
	return *this += buf;
	}

string& string::operator>>( int& i ) const {
	sscanf(((char*)*this),"%d",&i);
	return *((string*)this);
	}

int string::operator==( const string& a ) const {
	return !strcmp((char*)*((string*)this),(char*) a);
	}
 
int string::operator==( const char* a ) const {
        return !strcmp((char*)*((string*)this), a);
        }

int string::compare( const string& a, int opt ) const {
	if (opt == 1)
            return !strcasecmp((char*)*this,(char*) a);
        else
	    return !strcmp((char*)*((string*)this),(char*) a);
        }

int string::compare( const char* a, int opt ) const {
	if (opt == 1)
	    return !strcasecmp((char*)*((string*)this), a);
	else
	    return !strcmp((char*)*((string*)this), a);
	}

int string::operator!=( const string& a ) const {
	return strcmp((char*)*((string*)this),(char*) a);
	}

int string::operator!=( const char* a ) const {
	return strcmp((char*)*((string*)this), a);
	}


int string::operator<=( const string& a ) const {
	return ((*this) == a) || ((*this) < a);
	}

int string::operator<=( const char* a ) const {
	return ((*this) == a) || ((*this) < a);
	}


int string::operator<( const string& a ) const { 
	//return (*this) < (char*) a;
	return this->operator<((char*) a);
	}

int string::operator<( const char* a ) const { 
	char* p = (char*) (*this);
	int b = 1;
	//if (strlen(p) > strlen(a)) return 0;
	while ( *p != '\0' && *a != '\0' && b ) {
		char c1 = tolower(*p);
		char c2 = tolower(*a);
		if (c1 < c2) return b = 1;
		else if (c1 > c2) return b = 0;
		p++; a++;
		}
	//if ( *a == '\0' && *p != '\0' ) b = 0;
	return b;
	}

int string::substr( const char* str ) const {
	char* s = strstr(REP->rep,str);
	if (s) return (s - REP->rep);
	else return -1;
	}

int string::match( const char* str, int opt ) const {
#ifndef NOREGEX
	dvregex r(str,opt);
	return r.match( REP->rep );
#else
	return 0;
#endif
	}

int string::split(const char* sp) {
	REP->split((char*)sp);
	return  (REP->_list ? REP->_list->length(): 0);
	}

int string::matches(const char* str, int opt) {
	cell* tmp = 0;
#ifndef NOREGEX
	regex r(str,opt);
	if (!r.match( REP->rep )) return 0;
	range rng;
	for (int i=0; i<r.groups(); i++) {
		rng = r.getgroup(i);
		int st = rng.start(); int nd = rng.length();
		// *handler::_err << "from: " << st << " len: " << nd << endl;
		string* s = new string( _substring(st,nd) );
		_register(s);
		if (!tmp) tmp = new cell(s);
		else tmp->insert( s );
		}
#endif
      if (tmp) {
	if (REP->_list) delete REP->_list;
	REP->_list = tmp;
	return tmp->length();
      } else {
	return 0;
      }
}

void string::_register(iter<string>* p) const {
	REP->i_register(p); 
	}

void string::_register(const string* p) const {
	REP->s_register((string*)p); 
	}

void string::_register(const substring* p) const {
	REP->u_register((substring*)p); 
	}

string_iter::string_iter(stringrep* r)
			: _r(r), iter<string>(this) {
	n = 0;
	}

string* string_iter::operator()() {
	string* sp = 0; 
	if ( _r->_argc > n && _r->_argn == 0 ) {
		sp = new string( _r->_argv[n] );
		_r->s_register(sp);
		n++;
		}
	else if ( _r->_argc > n && _r->_argn != 0 ) {
		char buf[BUFSIZ];
		sprintf(buf,"%s=%s", _r->_argn[n], _r->_argv[n]);
		sp = new string( buf );
		_r->s_register(sp);
		n++;
		}
	return sp;
	}




string::operator iter<string>&() {

	iter<string>* result = 0;
	if (REP->_list) {
		result =  (iter<string>*) new celliter(REP->_list);
		}
	else if (((string*)this)->argc() > 0) {
	   result =  (iter<string>*) new string_iter(REP); 
	} 
	else result = new iter<string>(0);
	_register(result);
	return *result;
}


int string::argc() {
	   if (! REP->_mode) unflatten();
	   return REP -> _argc; 
	   }

char** string::argv( int i ) {
	   if (! REP->_mode) unflatten();
	   if (i > REP->_argc)
	      return ( REP -> _argv ) + i;
	   else return 0;
	   }

string string::arg( int i ) {
	    if (! REP->_mode) unflatten();
	    if (i < REP->_argc) 
	    return REP -> _argv[i]; 
	    else return "";
	    }

string& string::flatten(int i) {
	    if (REP->_mode != string::argcv )
	       ;//AE *handler::_err << "WARNING: flatten not allowed" << endl;
	    else REP->flatten(i);
	    return *this;
	    }

char** string::convert() {
	    REP->convert();
	    return REP->_argv;
	    }

int string::unflatten() { return REP->unflatten(); }

int string::isassoc() { return REP->isassoc(); }
string string::key() { return isassoc()?REP->_key:""; }
string string::value() { return isassoc()?REP->_value:""; }

static int check(const char* s) {
int b = 0; const char* p = s;
while (!b && *p) if (*p++ == ' ') b = 1;
return b;
}

void stringrep::s_register(string* p) { REGISTER(_garb,p); }
void stringrep::l_register(cell* p) { REGISTER(_lgarb,p); }
void stringrep::u_register(substring* p) { REGISTER(_ugarb,p); }
void stringrep::i_register(iter<string>* p) { REGISTER(_igarb,p); }



stringrep::stringrep(const char* s) : _mode(string::plain) {
	if (s && (((void*)s) != (void*) 0x1)) {
	  rep = new char[strlen(s)+1]; strcpy(rep,s);
	} else rep=0;
	refs = 0; _key = 0; _value = 0; _list = 0;
	_argn = 0; _argv = 0;
	_garb = 0; _igarb = 0; _lgarb = 0; _ugarb = 0;
	len = 0;
	}

stringrep::stringrep( int argc_, char** argv_) : _mode(string::argcv) {
refs = 0; _key = 0; _value = 0;
_argc = argc_;
_argv = argv_;
_argn = 0;
rep = 0;
_list = 0;
_garb = 0; _igarb = 0; _lgarb = 0; _ugarb = 0; len = 0;
}


stringrep::stringrep( int argc_, char** argn_, char** argv_) : _mode(string::argcv) {
refs = 0; _key = 0; _value = 0;
_argc = argc_;
_argv = argv_;
_argn = argn_;
rep = 0;
_list = 0;
_garb = 0; _igarb = 0; _lgarb = 0; _ugarb = 0; len = 0;
}

stringrep::~stringrep() { 
	  if (rep) delete rep; 
	  if (_mode == string::pair) {
		if (_key) delete _key;
		if (_value) delete _value;
		}
	  if (_list) delete _list;
	  HUSHDELETE(_garb,string);
	  HUSHDELETE(_lgarb,cell);
	  HUSHDELETE(_igarb,iter<string>);
	  HUSHDELETE(_ugarb,substring);
	  }     


int stringrep::length() {
	if (!len) len = strlen(rep);
	return len;
	}

	

void stringrep::convert() {
	int n = 0;
	while ( n < _argc ) {
                char buf[BUFSIZ];
                sprintf(buf,"%s=%s", _argn[n], _argv[n]);
                string* sp = new string( buf );
                s_register(sp);
		_argv[n] = (char*) sp;
                n++;
                }
        }



char* stringrep::flatten( int b ) {
int argc_ = _argc;
char** argv_ = _argv;
char *temp; 
int n = BUFSIZ;
int m = 0;
temp = new char[n];
temp[0]='\0';
if (argc_ > 0)  { m = strlen(*argv_); strcpy(temp,*argv_++); argc_--; }
while (--argc_ >= 0) {
        m += strlen(*argv_);
	if ( m > n ) {
		char* tn = new char[n]; 
		strcpy(tn,temp);
		n *= 2;
		delete temp;
		temp = new char[n];
		strcpy(temp,tn);
		delete tn;
		}
	if (b && check(*argv_)) 
	    sprintf(temp,"%s {%s}",temp,*argv_);
	else
	    sprintf(temp,"%s %s",temp,*argv_);
	argv_++;
        }
rep = new char[strlen(temp)+1]; strcpy(rep,temp);
delete temp;
return rep;
}






int stringrep::unflatten() {
char* in = rep;
static char argv_[MaxWArgs][MaxLArgs];
_argv = (char**) argv_;
int brackets = 0;
int quoteon = 0; int cont = 1;
char inbuf[BUFSIZ];
strcpy(inbuf,in);
char* b = inbuf;
while (*b == ' ') b++;
char* p = b;
int i = 0;
while ( *p != '\0' ) {
i++;
char prev = '\0';
quoteon = 0; cont = 1; /// quotes[quoteon]
while ( *p != '\0' && cont ) {
	if ( *p == ' ' && !quoteon && !brackets ) cont = 0;
	else if ( *p == '"' && prev != '\\') quoteon = quoteon?0:1;
	else if ( *p == '{' && prev != '\\') brackets++;
	else if ( *p == '}' && prev != '\\') brackets--;
	if (cont) { prev = *p;  p++; }
	}

if ( *p == '\0' ) {
	strcpy(argv_[i-1],b);
	return i;
} else {
	*p = '\0';
	strcpy(argv_[i-1],b);
	p++;
	b = p;
	}
}
_argc = i;
return i;
}


int stringrep::isassoc() {
if (_mode == string::pair) return 1;
else {
  int res = 0;
  for ( char* p = rep; *p ; p++ ) {
	if (*p == '=') res = 1;
	}
  if (res) mkassoc();
  return res;
  }
}



void stringrep::mkassoc() {
int m = strlen(rep);
char* _buf = new char[m+1];
strcpy(_buf,rep);
char* p = _buf;
while ( *p != '=' ) p++;
*p = '\0';
if ( *(p+1) == '"' ) p++;
_key = new char[strlen(_buf)+1];
_key[0]='\0';
strcpy(_key,_buf);
_value = new char[strlen(p+1)+1];
strcpy(_value,p+1);
int n = strlen( _value ) - 1;
if (_value[n]=='"') _value[n]='\0';
_mode = string::pair;
}

static int charin(char c, const char* in) {
char* p = (char*) in;
while ( *p ) {
	if ( c == *p ) return 1;
	p++;
	}
return 0;
}


void stringrep::split(const char* s) {
if (_list) delete _list;
_list = 0;
char* p = rep?rep:0;

if (p) while (*p) {
	char buf[BUFSIZ];
	char* q = buf;
	*q = '\0';

	while ( *p && !charin(*p,s) ) {
		*q = *p; p++; q++;
		}
	if (*p) while( charin(*p,s) ) p++;
	*q = '\0';
	string* tmp = new string( buf );
	s_register(tmp);
	if (!_list) _list = new cell(tmp);
	else _list->insert(tmp);
	}
}


#include <ctype.h> // for tolower
string& string::lower() const // how to do this efficient?
        {
                 char    *t = new char[length()+1];
                 int     i=0;
      
		 char* p = (char*) *this;
		 int n = length();
                 for(i = 0; i < n; i++)
                         t[i] = tolower(*p++);
                 t[i] = '\0';
                 string* tmp = new string(t); // memory leak
		 _register( tmp ); // leak fixed
                 delete t;
                 return *tmp;
}

string& string::upper() const // how to do this efficient?
        {
                 char    *t = new char[length()+1];
                 int     i=0;
      
		 char* p = (char*) *this;
		 int n = length();
                 for(i = 0; i < n; i++)
                         t[i] = toupper(*p++);
                 t[i] = '\0';
                 string* tmp = new string(t); // memory leak
		 _register( tmp ); // leak fixed
                 delete t;
                 return *tmp;
}



