/*
*
* Old Camera Effect
* Created by 
*     Arkadiusz Waliszewski
*     arsil@acm.org
*

<scene_commands>
	<setcolortarget texture="SceneMap"/>
	<setdepthstenciltarget texture="DepthMap"/>
	<clear color="0,0,0" depth="1.0"/>
	<draw type="objects"/>
	<setcolortarget/>
	<settechnique name="OldCameraPostprocessing"/>
	<draw type="buffer"/>
</scene_commands>

*/

texture SceneMap : RENDERCOLORTARGET < 
	float2 ViewPortDimensions = {1.0, 1.0};
	int MIPLEVELS = 1;
	string format = "X8R8G8B8";
	string UIWidget = "None";
>;

texture DepthMap : RENDERDEPTHSTENCILTARGET
<
	float2 ViewPortDimensions = {1.0,1.0};
	string format = "D24S8";
	string UIWidget = "None";
>;

sampler SceneSamp = sampler_state 
{
	texture = <SceneMap>;
	AddressU  = CLAMP;        
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = NONE;
	MINFILTER = NONE;
	MAGFILTER = NONE;
};
//----------------------------------------------------------------------------

//! dust texture
texture DustTexture
<
	string ResourceName="dust.dds";
>;
//----------------------------------------------------------------------------

//! dust sampler
sampler DustSamp = sampler_state 
{
	texture = <DustTexture>;
	AddressU  = BORDER;        
	AddressV  = BORDER;
	AddressW  = CLAMP;
	MIPFILTER = NONE;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
	BorderColor = {1.0, 1.0, 1.0, 1.0};
};
//----------------------------------------------------------------------------

//! line texture
texture LineTexture
<
	string ResourceName="line.dds";
>;
//----------------------------------------------------------------------------

//! line sampler
sampler LineSamp = sampler_state 
{
	texture = <LineTexture>;
	AddressU  = BORDER;        
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = NONE;
	MINFILTER = NONE;
	MAGFILTER = NONE;
	BorderColor = {1.0, 1.0, 1.0, 1.0};
};
//----------------------------------------------------------------------------

//! tv texture
texture TvAndNoiseTexture
<
	string ResourceName="Tv.dds";
>;
//----------------------------------------------------------------------------

//! tv sampler
sampler TvSamp = sampler_state 
{
	texture = <TvAndNoiseTexture>;
	AddressU  = CLAMP;        
	AddressV  = CLAMP;
	AddressW  = CLAMP;
	MIPFILTER = NONE;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};
//----------------------------------------------------------------------------

//! noise sampler
sampler NoiseSamp = sampler_state 
{
	texture = <TvAndNoiseTexture>;
	AddressU  = WRAP;        
	AddressV  = WRAP;
	AddressW  = WRAP;
	MIPFILTER = NONE;
	MINFILTER = LINEAR;
	MAGFILTER = LINEAR;
};
//----------------------------------------------------------------------------


//! current time
half Timer : TIME < string UIWidget = "None"; >; 
//----------------------------------------------------------------------------

//! window size
half2 WindowSize : VIEWPORTPIXELSIZE < string UIWidget = "None"; >;  
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Effect controls
//----------------------------------------------------------------------------

half OverExposureAmount <
	string UIName = "Overexposure amount";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 10.0;
	half UIStep = 0.1;
> = 1.5;
//----------------------------------------------------------------------------

half DustAmount <
	string UIName = "Dust amount";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 4.0;
	half UIStep = 1;
> = 4.0;
//----------------------------------------------------------------------------

half FrameJitter <
	string UIName = "Frame Jitter Frequency";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 6.0;
	half UIStep = 0.1;
> = 4.7;
//----------------------------------------------------------------------------

half MaxFrameJitter <
	string UIName = "Max Frame Jitter Amount";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 10.0;
	half UIStep = 0.1;
> = 1.4;
//----------------------------------------------------------------------------

half4 FilmColor <
	string UIWidget = "Color";
> = {1.0, 0.7559052, 0.58474624, 1.0};
//----------------------------------------------------------------------------

half GrainThicknes <
	string UIName = "Grain Thicknes";
	string UIWidget = "slider";
	half UIMin = 0.1;
	half UIMax = 4.0;
	half UIStep = 0.1;
> = 1.0;
//----------------------------------------------------------------------------

half GrainAmount <
	string UIName = "Grain Amount";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 1.0;
	half UIStep = 0.1;
> = 0.8;
//----------------------------------------------------------------------------

half ScratchesAmount <
	string UIName = "Num vert. scratches";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 3.0;
	half UIStep = 1;
> = 3.0;
//----------------------------------------------------------------------------

half ScratchesLevel <
	string UIName = "Num vert. scratches";
	string UIWidget = "slider";
	half UIMin = 0.0;
	half UIMax = 1.0;
	half UIStep = 0.1;
> = 0.7;
//----------------------------------------------------------------------------

//! vertex shader output/pixel shader input
struct VS_OUTPUT
{
	half4 Position        : POSITION;
	half4 SceneCoord      : TEXCOORD0;

	half4 Dust0And1Coords : TEXCOORD1;
	half4 Dust2And3Coords : TEXCOORD2;

	half4 TvCoords        : TEXCOORD3;
	half4 NoiseCoords     : TEXCOORD4;

	half4 Line0VertCoords : TEXCOORD5;
	half4 Line1VertCoords : TEXCOORD6;
	half4 Line2VertCoords : TEXCOORD7;

	half4 OverExposure    : DIFFUSE;
};
//----------------------------------------------------------------------------

//! vertex shader
VS_OUTPUT VS_Quad(
	half3 Position : POSITION, 
	half3 TexCoord : TEXCOORD0)
{
	// some pseudo-random numbers
	float Random0 = Timer % 7.000;
	float Random1 = Timer % 3.300;
	float Random2 = Timer % 0.910;
	float Random3 = Timer % 0.110;
	float Random4 = Timer % 0.070;
	float Random5 = Timer % 0.013;

	VS_OUTPUT OUT = (VS_OUTPUT)0;

	// pass 
	OUT.Position = half4(Position, 1);

	half2 inverseWindowSize = half2(1.0, 1.0) / WindowSize;

	// compute vert. frame jitter
	float frameJitterY =  40 * MaxFrameJitter * Random2 * Random0 * Random3;
	if (frameJitterY < (6 - FrameJitter) * 10)
		frameJitterY = 0;
 	frameJitterY *=inverseWindowSize.y;

	// add jitter to the original coords.
	OUT.SceneCoord =  half4(TexCoord, 1) + half4(0, frameJitterY, 0,0);

	// compute overexposure amount
	OUT.OverExposure = OverExposureAmount * Random3;
	
	// pass original screen coords (border rendering)
	OUT.TvCoords = half4(TexCoord, 1);

	// compute noise coords.
	half2 NoiseCoordsTmp = (WindowSize / (GrainThicknes * half2(128.0, 128.0)))
		* TexCoord.xy;
	NoiseCoordsTmp += half2(100 * Random3 * Random1 - Random0, 
		Random4 + Random1 * Random2);
	OUT.NoiseCoords = NoiseCoordsTmp.xyxy;

	// dust section (turns on or off particular dust texture)
	if (DustAmount > 0)
	{
		OUT.Dust0And1Coords.xy = 2.0 * Position.xy 
			+ 200 * float2(Random1 * Random4, Timer % 0.03);
	}
	else
	{
		OUT.Dust0And1Coords.xy = 0;
	}

	if (DustAmount > 1)
	{
		OUT.Dust0And1Coords.zw = 2.3 * Position.yx 
			- 210 * float2(Random4 * 0.45, Random5 * 2);
	}
	else
	{
		OUT.Dust0And1Coords.zw = 0;
	}

	if (DustAmount > 2)
	{
		OUT.Dust2And3Coords.xy = 1.4 * Position.xy 
			+ float2(700, +100) * float2(Random2 * Random4, Random2);
	}
	else
	{
		OUT.Dust2And3Coords.xy = 0;
	}

	if (DustAmount > 3)
	{
		OUT.Dust2And3Coords.zw = 1.7 * Position.yx 
			+ float2(-100, 130) * float2(Random2 * Random4, Random1 * Random4);
	}
	else
	{
		OUT.Dust2And3Coords.zw = 0;
	}
	
	// vert lines section
	OUT.Line0VertCoords   = 0.5 * Position.xxxx * WindowSize.xxxx * 0.3;
	OUT.Line1VertCoords = OUT.Line0VertCoords;
	OUT.Line2VertCoords = OUT.Line0VertCoords;

	// first line
	if (ScratchesAmount > 0)
	{
		OUT.Line0VertCoords.x += 0.15 - ((Random1 - Random3 + Random2) * 0.1) 
			* WindowSize.x;
	}
	else
	{
		OUT.Line0VertCoords.x = -1;
	}

	// second line
	if (ScratchesAmount > 1)
	{
		OUT.Line1VertCoords.x += 0.55 + ((Random0 - Random2 + Random4) * 0.1) 
			* WindowSize.x;
	}
	else
	{
		OUT.Line1VertCoords.x = -1;
	}

	// third line
	if (ScratchesAmount > 2)
	{
		OUT.Line2VertCoords.x += 0.31 + ((Random1 - Random2 + Random4) * 0.2) 
			* WindowSize.x;
	}
	else
	{
		OUT.Line2VertCoords.x = -1;
	}


	return OUT;
}
//----------------------------------------------------------------------------

//! pixel shader
half4 PS_TV(VS_OUTPUT IN) : COLOR
{   
	// sample scene
	half4 img = tex2D(SceneSamp, IN.SceneCoord.xy);

	// compute sepia (or other color) image
	half4 img2 = dot(img, half4(0.30, 0.59, 0.11, 0.3)) 
		* FilmColor;

	// sample dust textures
	half4 dust0 = tex2D(DustSamp, IN.Dust0And1Coords.xy);
	half4 dust1 = tex2D(DustSamp, IN.Dust0And1Coords.wz);
	half4 dust2 = tex2D(DustSamp, IN.Dust2And3Coords.xy);
	half4 dust3 = tex2D(DustSamp, IN.Dust2And3Coords.wz);

	// sample line textures
	half4 line0 = tex2D(LineSamp, IN.Line0VertCoords.xy);
	half4 line1 = tex2D(LineSamp, IN.Line1VertCoords.xy);
	half4 line2 = tex2D(LineSamp, IN.Line2VertCoords.xy);

	// sample border texture
	half4 tv = tex2D(TvSamp, IN.TvCoords.xy);

	// sample noise values
	half4 noiseVal = tex2D(NoiseSamp, IN.NoiseCoords.xy);
	noiseVal = lerp(half4(1.0, 1.0, 1.0, 1.0), noiseVal, GrainAmount);

	// "accumulate" dust
	half finalDust = dust0.x * dust1.y * dust2.z * dust3.w;

	// "accumulate" lines
	half finalLines = line0.x * line1.x * line2.x;
	finalLines = lerp(half4(1.0, 1.0, 1.0, 1.0), finalLines, ScratchesLevel);
	
	// final composition
	return tv * (IN.OverExposure + finalDust * finalLines * img2)
		* noiseVal.wwww;
}
//----------------------------------------------------------------------------

technique OldCameraPostprocessing
{
	pass TV_lines
	{
		cullmode = none;
		ZEnable = false;
		VertexShader = compile vs_2_0 VS_Quad();
		PixelShader  = compile ps_2_0 PS_TV();
	}
}
