// ViP (AE)

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 ambient;	// Material's ambient color
float4 diffuse;	// Material's diffuse color

// This effect file uses a single directional light
float3 direction = normalize(float3(1.0f, 1.0f, 1.0f));	// Light's direction in world space
float4 ambient_light = { 1.0f, 1.0f, 1.0f, 1.0f };			// Light's ambient color
float4 diffuse_light = { 1.0f, 1.0f, 1.0f, 1.0f };			// Light's diffuse color

float4 global = { 0.5f, 0.5f, 3.0f, 0.5f};	

texture tex;			// Color texture for mesh

float	 time;					// App's time in seconds
float4x4 world;					// World matrix for object
float4x4 wvp;	// World * View * Projection matrix



//-----------------------------------------------------------------------------
// texture samplers
//-----------------------------------------------------------------------------

sampler tex_sampler = 
sampler_state
{
    Texture = <tex>;    
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 position   : POSITION;   // vertex position 
    float4 diffuse    : COLOR0;     // vertex diffuse color
    float2 textureUV  : TEXCOORD0;  // vertex texture coords 
};


//-----------------------------------------------------------------------------
// Name: RenderSceneVS
// Type: Vertex shader                                      
// Desc: This shader computes standard transform and lighting
//-----------------------------------------------------------------------------

VS_OUTPUT RenderSceneVS( float4 pos : POSITION, 
                         float3 normal : NORMAL,
                         float2 texc0 : TEXCOORD0 )
{
    VS_OUTPUT output;
    float3 vnorm;
  
	// Animation the vertex based on time and the vertex's object space position
    float4 apos = pos + float4( sin(time) * pos.x/2, 0, 0, 0 );
    
    // Transform the position from object space to homogeneous projection space
    output.Position = mul(vAnimatedPos, wvp);
    
    // Transform the normal from object space to world space	
    vnorm = normalize(mul(normal, (float3x3)world)); // normal (world space)
    
    // Compute simple directional lighting equation
    output.diffuse.rgb = diffuse * diffuse_light * max(0,dot(vnorm, direction)) + 
                         ambient * ambient_light;  
                    
    output.diffuse.a = 0.1f; //1.0f; 
    output.diffuse.r *= 2.0f;
    //Output.Diffuse.a = 0.5f; //1.0f;
    
    
    // Just copy the texture coordinate through
    output.textureUV = texc0; 
    
    return output;    
}


//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 clr : COLOR0;  // Pixel color    
};


//-----------------------------------------------------------------------------
// Name: RenderScenePS                                        
// Type: Pixel shader
// Desc: This shader outputs the pixel's color by modulating the texture's
//		 color with diffuse material color
//-----------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT in ) 
{ 
    PS_OUTPUT output;

    // Lookup mesh texture and modulate it with diffuse
    output.clr = tex2D(tex_sampler, in.textureUV) * global; //In.Diffuse;
    //Output.RGBColor = In.Diffuse;

    return output;
}



//-----------------------------------------------------------------------------
// Name: RenderScene
// Type: Technique                                     
// Desc: Renders scene to render target
//-----------------------------------------------------------------------------

technique RenderScene
{
    pass P0
    {          
        VertexShader = compile vs_1_1 RenderSceneVS();
        PixelShader  = compile ps_1_1 RenderScenePS(); // trival pixel shader (could use FF instead if desired)
    }
}
