//**************************************************************//
//  Effect File exported by RenderMonkey
//
//  - Although many improvements were made to RenderMonkey FX  
//    file export, there are still situations that may cause   
//    compilation problems once the file is exported, such as  
//    occasional naming conflicts for methods, since FX format 
//    does not support any notions of name spaces. You need to 
//    try to create workspaces in such a way as to minimize    
//    potential naming conflicts on export.                    
//    
//  - Note that to minimize resulting name collisions in the FX 
//    file, RenderMonkey will mangle names for passes, shaders  
//    and function names as necessary to reduce name conflicts. 
//**************************************************************//
//--------------------------------------------------------------//
// Morph
//--------------------------------------------------------------//
//--------------------------------------------------------------//
// Single Pass
//--------------------------------------------------------------//
string Morph_Single_Pass_Tesselated_Cube : ModelData = "C:/Program Files/ATI Research Inc/RenderMonkey 1.5/Examples/Media/Models/TesselatedCube.3ds";

float4x4 view_proj_matrix : ViewProjection;
float4x4 view_matrix : View;
float time_0_X : Time0_X;
float lerpMin
<
   string UIName = "lerpMin";
   string UIWidget = "Numeric";
   float UIMin = -4.00;
   float UIMax = 4.00;
> = -2.00;
float lerpMax
<
   string UIName = "lerpMax";
   string UIWidget = "Numeric";
   float UIMin = -4.00;
   float UIMax = 4.00;
> = 2.00;
float speed
<
   string UIName = "speed";
   string UIWidget = "Numeric";
   float UIMin = 0.00;
   float UIMax = 1.00;
> = 0.11;
// This shader performs a morphing between a cube and a sphere,
// like in the classical screen-saver. The source model is a
// cube with a fair amount of tesselation. The sphere is generated
// simply by normalizing the cube's vertices. Morphing between the
// keyframes are done with simple linear interpolation.

struct VS_OUTPUT {
   float4 Pos:     POSITION;
   float3 normal:  TEXCOORD0;
   float3 viewVec: TEXCOORD1;
   float4 base:    TEXCOORD2;
};

VS_OUTPUT Morph_Single_Pass_Vertex_Shader_main(float4 Pos: POSITION, float3 normal: NORMAL){
   VS_OUTPUT Out;

   // Define the two key frames
   float3 spherePos = normalize(Pos.xyz);
   float3 cubePos = 0.9 * Pos.xyz;

   float3 sphereNormal = spherePos;
   float3 cubeNormal = normal;

   // Make a smooth 0->1->0 curve
   float t = frac(speed * time_0_X);
   t = smoothstep(0, 0.5, t) - smoothstep(0.5, 1, t);

   // Find the interpolation factor
   float lrp = lerpMin + (lerpMax - lerpMin) * t;

   // Linearly interpolate the position and normal
   Pos.xyz = lerp(spherePos, cubePos, lrp);
   normal = lerp(sphereNormal, cubeNormal, lrp);

   // Use position as base color
   Out.base = 0.5 + 0.5 * Pos;
   // Give the thing some size
   Pos.xyz *= 30;

   Out.Pos = mul(view_proj_matrix, Pos);
   // Eye-space lighting
   Out.normal = mul(view_matrix, normal);
   Out.viewVec = -mul(view_matrix, Pos);

   return Out;
}


float4 lightDir
<
   string UIName = "lightDir";
   string UIWidget = "Direction";
   float4 UIMin = ( -10.00, -10.00, -10.00, -10.00 );
   float4 UIMax = ( 10.00, 10.00, 10.00, 10.00 );
   bool   Normalize = false;
> = ( 0.41, 0.41, -0.82, 1.00 );
float4 Morph_Single_Pass_Pixel_Shader_main(float3 normal: TEXCOORD0, float3 viewVec: TEXCOORD1, float4 base: TEXCOORD2) : COLOR {
   normal = normalize(normal);

   // Soft diffuse
   float diffuse = 0.5 + 0.5 * dot(lightDir, normal);
   // Standard specular
   float specular = pow(saturate(dot(reflect(-normalize(viewVec), normal), lightDir)), 24);

   return diffuse * base + specular;
}



















//--------------------------------------------------------------//
// Technique Section for Morph
//--------------------------------------------------------------//
technique Morph
{
   pass Single_Pass
   {
      VertexShader = compile vs_1_1 Morph_Single_Pass_Vertex_Shader_main();
      PixelShader = compile ps_2_0 Morph_Single_Pass_Pixel_Shader_main();
   }

}

