
#ifndef _DICTIONARY_H
#define _DICTIONARY_H

#include <hush/export.h> // can this be refined?

#include <hush/iter.h>
#include <hush/string.h>
#include <hush/garbage.h>



// see man:dictionary(4),  //examples/hush/dictionary.h

class hidden_dictionary_handler {
public:
virtual int operator()() { return 0; }
virtual void _put(void*)  {           }
};


class dvstring;

class HUSH_API dictionary_type {
public:
dictionary_type(); //{ created++; }
virtual ~dictionary_type(); //{ deleted++; }
static int created;
static int deleted;

static void* table;
static void declare(const char*, void*, int flag = 0);
static dictionary_type* lookup(const char*);
static void _destroy();
};

class HUSH_API dictionary_api {
public:
static void set_dictionaryentry(void* raw, char* k, void* v);
static void* get_dictionaryentry(void* raw, char* p);
};

//static dictionaryentry<void> dummy;


template<class T>
class dictionaryentry {
public:
inline dictionaryentry( void* raw, char* k) : _k(k), _raw(raw){ }
inline T* operator=( T* v ) {
	dictionary_api::set_dictionaryentry(_raw,_k,(void*)v); 
	return (T*) v;
	}
inline operator T*() {
	return (T*) ((_k&&_raw)?dictionary_api::get_dictionaryentry(_raw,_k):0);
	}

private:
handler* h;
char* _k;
void* _raw;
};


class HUSH_API rawdictionary {
friend void  dictionary_api::set_dictionaryentry(void* raw, char* k, void* v);
friend void*  dictionary_api::get_dictionaryentry(void* raw, char* k);
public:
rawdictionary();
~rawdictionary();
void entry(char* k, void* v);
void* find(char* k);
void* del(char* k);
void destroy();

char* stats();
	
iter<dvstring>* search();
void* operator()(char* k);

dictionaryentry<void>* operator[](char* k);

void _register(void*);
void s_register(void*);
void e_register(void*);
void i_register(void*);

hidden_dictionary_handler* _auto;
	
protected:
void* tbl;
void* _garb; /// garbage
void* _egarb; /// entry
void* _sgarb; /// string
void* _igarb; /// iter
void* _lgarb; /// lists ?
};


//// The dictionary class:

static void* _result; // to avoid ansi c++ warnings

////#include <iostream.h> //AE!

template<class T>
class dictionary : public dictionary_type {
public:
	class auto_handler : public hidden_dictionary_handler {
	dictionary<T>* _d;
	T* _v;
	public:
         inline  auto_handler(dictionary<T>* d, T* v = 0) {
		_d = d; _v = v;
		}
	  inline void _put(void *x) {
		_v = (T*) x; 
		}
	  inline int operator()() {
		if (_v) _d->_register(_v); return 0; 
		}
	};

public: //@ dictionary<T>
inline dictionary( int i = 0) { 
	_auto = i; _garb = 0; d = new rawdictionary();
	}
inline ~dictionary() { delete d; if (_garb) delete _garb; }

inline void entry(char* k, T* v) { 
	if (k && v) d->entry(k,v);
	if (_auto && v) _register(v);
	}

inline T* find(char* k) { return (T*) (k?d->find(k):0); }
inline  T* del(char* k) { return (T*) (k?d->del(k):0); }

inline void destroy() { d->destroy(); }

inline char* stats() { return d->stats(); }
	
//T* operator()(char* k) { return (T*) d->operator()(k); }

inline operator iter<dvstring>&() { return *(iter<dvstring>*) d->search(); }
inline iter<dvstring>& traverse() { return *(iter<dvstring>*) d->search(); }

inline dictionaryentry<T>& operator[](dvstring* k) { 
	if (!k) return (*this)[(char*) 0];
	return (*this)[(char*)*k];
	}

inline  dictionaryentry<T>& operator[](const char* k) {
	_result = 0;
	if (k) _result = (*d)[ (char*) k];
	if (_auto && d) d->_auto = new auto_handler(this);
	return (dictionaryentry<T>&) * (dictionaryentry<T>*) _result;
	}

inline void _register(const T* x) const {
		if (!_garb) ((dictionary<T>*)this)->_garb =
						new dvgarbage<T>;
		_garb->_register(x);
		}
	
public:
rawdictionary* d;
dvgarbage<T>* _garb;
public:
int _auto; // careful


static inline void declare(const char* s, void* h, int flag = 0) {
        dictionary_type::declare(s,h,flag);
        }
 
static inline dictionary<T>* lookup(const char* s) {
        return (dictionary<T>*) dictionary_type::lookup(s);
        }
 

};

#endif
